@@ -39,7 +39,7 @@ static struct aws_error_info s_errors[] = {
         "Attempt to sign an http request with an invalid signing configuration"),
     AWS_DEFINE_ERROR_INFO_AUTH(
         AWS_AUTH_CREDENTIALS_PROVIDER_INVALID_ENVIRONMENT,
-        "Valid credentials could not be sourced from process environment"),
+        "Required environment variables could not be sourced from process environment"),
     AWS_DEFINE_ERROR_INFO_AUTH(
         AWS_AUTH_CREDENTIALS_PROVIDER_INVALID_DELEGATE,
         "Valid credentials could not be sourced from the provided vtable"),
@@ -105,7 +105,13 @@ static struct aws_error_info s_errors[] = {
         "Valid credentials could not be sourced by the sso credentials provider"),
     AWS_DEFINE_ERROR_INFO_AUTH(
         AWS_AUTH_IMDS_CLIENT_SOURCE_FAILURE,
-        "Failed to source the IMDS resource")
+        "Failed to source the IMDS resource"),
+            AWS_DEFINE_ERROR_INFO_AUTH(
+    AWS_AUTH_PROFILE_STS_CREDENTIALS_PROVIDER_CYCLE_FAILURE,
+        "Failed to resolve credentials because the profile contains a cycle in the assumeRole chain."),
+        AWS_DEFINE_ERROR_INFO_AUTH(
+    AWS_AUTH_CREDENTIALS_PROVIDER_ECS_INVALID_TOKEN_FILE_PATH,
+        "Failed to read the ECS token file specified in the AWS_CONTAINER_AUTHORIZATION_TOKEN_FILE environment variable."),
 };
 /* clang-format on */
 
