@@ -3,11 +3,13 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 
+#include "aws/common/byte_buf.h"
 #include <aws/auth/credentials.h>
 
 #include <aws/auth/private/credentials_utils.h>
 #include <aws/common/clock.h>
 #include <aws/common/date_time.h>
+#include <aws/common/environment.h>
 #include <aws/common/string.h>
 #include <aws/http/connection.h>
 #include <aws/http/connection_manager.h>
@@ -28,27 +30,23 @@
 #define ECS_RESPONSE_SIZE_LIMIT 10000
 #define ECS_CONNECT_TIMEOUT_DEFAULT_IN_SECONDS 2
 
+AWS_STATIC_STRING_FROM_LITERAL(s_ecs_creds_env_token_file, "AWS_CONTAINER_AUTHORIZATION_TOKEN_FILE");
+AWS_STATIC_STRING_FROM_LITERAL(s_ecs_creds_env_token, "AWS_CONTAINER_AUTHORIZATION_TOKEN");
+AWS_STATIC_STRING_FROM_LITERAL(s_ecs_creds_env_relative_uri, "AWS_CONTAINER_CREDENTIALS_RELATIVE_URI");
+AWS_STATIC_STRING_FROM_LITERAL(s_ecs_creds_env_full_uri, "AWS_CONTAINER_CREDENTIALS_FULL_URI");
+AWS_STATIC_STRING_FROM_LITERAL(s_ecs_host, "169.254.170.2");
+
 static void s_on_connection_manager_shutdown(void *user_data);
 
 struct aws_credentials_provider_ecs_impl {
     struct aws_http_connection_manager *connection_manager;
-    struct aws_auth_http_system_vtable *function_table;
+    const struct aws_auth_http_system_vtable *function_table;
     struct aws_string *host;
     struct aws_string *path_and_query;
+    struct aws_string *auth_token_file_path;
     struct aws_string *auth_token;
 };
 
-static struct aws_auth_http_system_vtable s_default_function_table = {
-    .aws_http_connection_manager_new = aws_http_connection_manager_new,
-    .aws_http_connection_manager_release = aws_http_connection_manager_release,
-    .aws_http_connection_manager_acquire_connection = aws_http_connection_manager_acquire_connection,
-    .aws_http_connection_manager_release_connection = aws_http_connection_manager_release_connection,
-    .aws_http_connection_make_request = aws_http_connection_make_request,
-    .aws_http_stream_activate = aws_http_stream_activate,
-    .aws_http_stream_get_incoming_response_status = aws_http_stream_get_incoming_response_status,
-    .aws_http_stream_release = aws_http_stream_release,
-    .aws_http_connection_close = aws_http_connection_close};
-
 /*
  * Tracking structure for each outstanding async query to an ecs provider
  */
@@ -58,6 +56,7 @@ struct aws_credentials_provider_ecs_user_data {
     struct aws_credentials_provider *ecs_provider;
     aws_on_get_credentials_callback_fn *original_callback;
     void *original_user_data;
+    struct aws_byte_buf auth_token;
 
     /* mutable */
     struct aws_http_connection *connection;
@@ -79,6 +78,7 @@ static void s_aws_credentials_provider_ecs_user_data_destroy(struct aws_credenti
             impl->connection_manager, user_data->connection);
     }
 
+    aws_byte_buf_clean_up(&user_data->auth_token);
     aws_byte_buf_clean_up(&user_data->current_result);
 
     if (user_data->request) {
@@ -95,9 +95,6 @@ static struct aws_credentials_provider_ecs_user_data *s_aws_credentials_provider
 
     struct aws_credentials_provider_ecs_user_data *wrapped_user_data =
         aws_mem_calloc(ecs_provider->allocator, 1, sizeof(struct aws_credentials_provider_ecs_user_data));
-    if (wrapped_user_data == NULL) {
-        goto on_error;
-    }
 
     wrapped_user_data->allocator = ecs_provider->allocator;
     wrapped_user_data->ecs_provider = ecs_provider;
@@ -109,12 +106,33 @@ static struct aws_credentials_provider_ecs_user_data *s_aws_credentials_provider
         goto on_error;
     }
 
+    struct aws_credentials_provider_ecs_impl *impl = ecs_provider->impl;
+    if (impl->auth_token_file_path != NULL && impl->auth_token_file_path->len > 0) {
+        if (aws_byte_buf_init_from_file(
+                &wrapped_user_data->auth_token,
+                ecs_provider->allocator,
+                aws_string_c_str(impl->auth_token_file_path))) {
+            AWS_LOGF_ERROR(
+                AWS_LS_AUTH_CREDENTIALS_PROVIDER,
+                "(id=%p) ECS credentials provider failed to read token from the path: %s with error: %d",
+                (void *)ecs_provider,
+                aws_string_c_str(impl->auth_token_file_path),
+                aws_last_error());
+            aws_raise_error(AWS_AUTH_CREDENTIALS_PROVIDER_ECS_INVALID_TOKEN_FILE_PATH);
+            goto on_error;
+        }
+    } else if (impl->auth_token != NULL && impl->auth_token->len > 0) {
+        if (aws_byte_buf_init_copy_from_cursor(
+                &wrapped_user_data->auth_token,
+                ecs_provider->allocator,
+                aws_byte_cursor_from_string(impl->auth_token))) {
+            goto on_error;
+        }
+    }
+
     return wrapped_user_data;
-
 on_error:
-
     s_aws_credentials_provider_ecs_user_data_destroy(wrapped_user_data);
-
     return NULL;
 }
 
@@ -148,7 +166,7 @@ static void s_ecs_finalize_get_credentials_query(struct aws_credentials_provider
     struct aws_credentials *credentials = NULL;
     struct aws_parse_credentials_from_json_doc_options parse_options = {
         .access_key_id_name = "AccessKeyId",
-        .secrete_access_key_name = "SecretAccessKey",
+        .secret_access_key_name = "SecretAccessKey",
         .token_name = "Token",
         .expiration_name = "Expiration",
         .token_required = true,
@@ -156,7 +174,7 @@ static void s_ecs_finalize_get_credentials_query(struct aws_credentials_provider
     };
     if (aws_byte_buf_append_null_terminator(&ecs_user_data->current_result) == AWS_OP_SUCCESS) {
         credentials = aws_parse_credentials_from_json_document(
-            ecs_user_data->allocator, (const char *)ecs_user_data->current_result.buffer, &parse_options);
+            ecs_user_data->allocator, aws_byte_cursor_from_buf(&ecs_user_data->current_result), &parse_options);
     } else {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
@@ -329,10 +347,10 @@ static int s_make_ecs_http_query(
         goto on_error;
     }
 
-    if (impl->auth_token != NULL) {
+    if (ecs_user_data->auth_token.len) {
         struct aws_http_header auth_header = {
             .name = aws_byte_cursor_from_string(s_ecs_authorization_header),
-            .value = aws_byte_cursor_from_string(impl->auth_token),
+            .value = aws_byte_cursor_from_buf(&ecs_user_data->auth_token),
         };
         if (aws_http_message_add_header(request, auth_header)) {
             goto on_error;
@@ -382,7 +400,10 @@ static int s_make_ecs_http_query(
         .user_data = ecs_user_data,
         .request = request,
     };
-
+    /* for test with mocking http stack where make request finishes
+      immediately and releases client before stream activate call */
+    struct aws_credentials_provider *provider = ecs_user_data->ecs_provider;
+    aws_credentials_provider_acquire(provider);
     stream = impl->function_table->aws_http_connection_make_request(ecs_user_data->connection, &request_options);
 
     if (!stream) {
@@ -392,6 +413,7 @@ static int s_make_ecs_http_query(
     if (impl->function_table->aws_http_stream_activate(stream)) {
         goto on_error;
     }
+    aws_credentials_provider_release(provider);
 
     return AWS_OP_SUCCESS;
 
@@ -442,6 +464,9 @@ static int s_credentials_provider_ecs_get_credentials_async(
     aws_on_get_credentials_callback_fn callback,
     void *user_data) {
 
+    AWS_LOGF_DEBUG(
+        AWS_LS_AUTH_CREDENTIALS_PROVIDER, "id=%p: ECS provider trying to load credentials", (void *)provider);
+
     struct aws_credentials_provider_ecs_impl *impl = provider->impl;
 
     struct aws_credentials_provider_ecs_user_data *wrapped_user_data =
@@ -470,6 +495,7 @@ static void s_credentials_provider_ecs_destroy(struct aws_credentials_provider *
 
     aws_string_destroy(impl->path_and_query);
     aws_string_destroy(impl->auth_token);
+    aws_string_destroy(impl->auth_token_file_path);
     aws_string_destroy(impl->host);
 
     /* aws_http_connection_manager_release will eventually leads to call of s_on_connection_manager_shutdown,
@@ -503,6 +529,12 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_ecs_options *options) {
 
+    if (!options->bootstrap) {
+        AWS_LOGF_ERROR(AWS_LS_AUTH_CREDENTIALS_PROVIDER, "ECS provider: bootstrap must be specified");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
     struct aws_credentials_provider *provider = NULL;
     struct aws_credentials_provider_ecs_impl *impl = NULL;
 
@@ -563,7 +595,7 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
 
     impl->function_table = options->function_table;
     if (impl->function_table == NULL) {
-        impl->function_table = &s_default_function_table;
+        impl->function_table = g_aws_credentials_provider_http_function_table;
     }
 
     impl->connection_manager = impl->function_table->aws_http_connection_manager_new(allocator, &manager_options);
@@ -576,6 +608,13 @@ struct aws_credentials_provider *aws_credentials_provider_new_ecs(
             goto on_error;
         }
     }
+    if (options->auth_token_file_path.len != 0) {
+        impl->auth_token_file_path = aws_string_new_from_cursor(allocator, &options->auth_token_file_path);
+        if (impl->auth_token_file_path == NULL) {
+            goto on_error;
+        }
+    }
+
     impl->path_and_query = aws_string_new_from_cursor(allocator, &options->path_and_query);
     if (impl->path_and_query == NULL) {
         goto on_error;
@@ -599,3 +638,106 @@ on_error:
 
     return NULL;
 }
+
+struct aws_credentials_provider *aws_credentials_provider_new_ecs_from_environment(
+    struct aws_allocator *allocator,
+    const struct aws_credentials_provider_ecs_environment_options *options) {
+
+    if (!options->tls_ctx) {
+        AWS_LOGF_ERROR(AWS_LS_AUTH_CREDENTIALS_PROVIDER, "ECS provider: tls_ctx must be specified");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
+    struct aws_credentials_provider_ecs_options explicit_options = {
+        .shutdown_options = options->shutdown_options,
+        .bootstrap = options->bootstrap,
+        .function_table = options->function_table,
+    };
+
+    struct aws_string *ecs_env_token_file_path = NULL;
+    struct aws_string *ecs_env_token = NULL;
+    struct aws_string *relative_uri_str = NULL;
+    struct aws_string *full_uri_str = NULL;
+    struct aws_uri full_uri;
+    AWS_ZERO_STRUCT(full_uri);
+    struct aws_credentials_provider *provider = NULL;
+
+    /* read the environment variables */
+    aws_get_environment_value(allocator, s_ecs_creds_env_token_file, &ecs_env_token_file_path);
+    aws_get_environment_value(allocator, s_ecs_creds_env_token, &ecs_env_token);
+    aws_get_environment_value(allocator, s_ecs_creds_env_relative_uri, &relative_uri_str);
+    aws_get_environment_value(allocator, s_ecs_creds_env_full_uri, &full_uri_str);
+
+    if (ecs_env_token_file_path != NULL && ecs_env_token_file_path->len > 0) {
+        explicit_options.auth_token_file_path = aws_byte_cursor_from_string(ecs_env_token_file_path);
+    }
+    if (ecs_env_token != NULL && ecs_env_token->len > 0) {
+        explicit_options.auth_token = aws_byte_cursor_from_string(ecs_env_token);
+    }
+
+    if (relative_uri_str != NULL && relative_uri_str->len != 0) {
+
+        /* Using RELATIVE_URI */
+        AWS_LOGF_INFO(
+            AWS_LS_AUTH_CREDENTIALS_PROVIDER,
+            "ECS provider: using relative uri %s",
+            aws_string_c_str(relative_uri_str));
+
+        explicit_options.path_and_query = aws_byte_cursor_from_string(relative_uri_str);
+        explicit_options.host = aws_byte_cursor_from_string(s_ecs_host);
+        explicit_options.port = 80;
+
+        provider = aws_credentials_provider_new_ecs(allocator, &explicit_options);
+
+    } else if (full_uri_str != NULL && full_uri_str->len != 0) {
+
+        /* Using FULL_URI */
+        AWS_LOGF_INFO(
+            AWS_LS_AUTH_CREDENTIALS_PROVIDER, "ECS provider: using full uri %s", aws_string_c_str(full_uri_str));
+        struct aws_byte_cursor full_uri_cursor = aws_byte_cursor_from_string(full_uri_str);
+        if (aws_uri_init_parse(&full_uri, allocator, &full_uri_cursor)) {
+            AWS_LOGF_ERROR(
+                AWS_LS_AUTH_CREDENTIALS_PROVIDER,
+                "ECS provider: Failed because %s environment variable is invalid uri %s.",
+                aws_string_c_str(s_ecs_creds_env_full_uri),
+                aws_string_c_str(full_uri_str));
+            goto cleanup;
+        }
+
+        explicit_options.host = *aws_uri_host_name(&full_uri);
+        explicit_options.path_and_query = *aws_uri_path_and_query(&full_uri);
+        if (explicit_options.path_and_query.len == 0) {
+            explicit_options.path_and_query = aws_byte_cursor_from_c_str("/");
+        }
+
+        if (aws_byte_cursor_eq_c_str_ignore_case(aws_uri_scheme(&full_uri), "https")) {
+            explicit_options.tls_ctx = options->tls_ctx;
+        }
+
+        explicit_options.port = aws_uri_port(&full_uri);
+        if (explicit_options.port == 0) {
+            explicit_options.port = explicit_options.tls_ctx ? 443 : 80;
+        }
+
+        provider = aws_credentials_provider_new_ecs(allocator, &explicit_options);
+
+    } else {
+        /* Neither environment variable is set */
+        AWS_LOGF_INFO(
+            AWS_LS_AUTH_CREDENTIALS_PROVIDER,
+            "ECS provider: Unable to initialize from environment because AWS_CONTAINER_CREDENTIALS_FULL_URI and "
+            "AWS_CONTAINER_CREDENTIALS_RELATIVE_URI are not set.");
+        aws_raise_error(AWS_AUTH_CREDENTIALS_PROVIDER_INVALID_ENVIRONMENT);
+        goto cleanup;
+    }
+
+cleanup:
+    aws_string_destroy(relative_uri_str);
+    aws_string_destroy(full_uri_str);
+    aws_string_destroy(ecs_env_token_file_path);
+    aws_string_destroy(ecs_env_token);
+
+    aws_uri_clean_up(&full_uri);
+    return provider;
+}
