@@ -7,15 +7,15 @@
 #include <aws/common/string.h>
 
 #include <aws/common/json.h>
-#include <aws/common/private/json_impl.h>
+#include <aws/common/private/external_module_impl.h>
 
-#include <aws/common/external/cJSON.h>
+#include "external/cJSON.h"
 
 static struct aws_allocator *s_aws_json_module_allocator = NULL;
 static bool s_aws_json_module_initialized = false;
 
 struct aws_json_value *aws_json_value_new_string(struct aws_allocator *allocator, struct aws_byte_cursor string) {
-    struct aws_string *tmp = aws_string_new_from_cursor((struct aws_allocator *)allocator, &string);
+    struct aws_string *tmp = aws_string_new_from_cursor(allocator, &string);
     void *ret_val = cJSON_CreateString(aws_string_c_str(tmp));
     aws_string_destroy_secure(tmp);
     return ret_val;
@@ -47,7 +47,7 @@ struct aws_json_value *aws_json_value_new_object(struct aws_allocator *allocator
 }
 
 int aws_json_value_get_string(const struct aws_json_value *value, struct aws_byte_cursor *output) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (!cJSON_IsString(cjson)) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
@@ -56,7 +56,7 @@ int aws_json_value_get_string(const struct aws_json_value *value, struct aws_byt
 }
 
 int aws_json_value_get_number(const struct aws_json_value *value, double *output) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (!cJSON_IsNumber(cjson)) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
@@ -65,7 +65,7 @@ int aws_json_value_get_number(const struct aws_json_value *value, double *output
 }
 
 int aws_json_value_get_boolean(const struct aws_json_value *value, bool *output) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (!cJSON_IsBool(cjson)) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
@@ -109,7 +109,7 @@ struct aws_json_value *aws_json_value_get_from_object(const struct aws_json_valu
     void *return_value = NULL;
     struct aws_string *tmp = aws_string_new_from_cursor(s_aws_json_module_allocator, &key);
 
-    struct cJSON *cjson = (struct cJSON *)object;
+    const struct cJSON *cjson = (const struct cJSON *)object;
     if (!cJSON_IsObject(cjson)) {
         aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
         goto done;
@@ -130,7 +130,7 @@ bool aws_json_value_has_key(const struct aws_json_value *object, struct aws_byte
     struct aws_string *tmp = aws_string_new_from_cursor(s_aws_json_module_allocator, &key);
     bool result = false;
 
-    struct cJSON *cjson = (struct cJSON *)object;
+    const struct cJSON *cjson = (const struct cJSON *)object;
     if (!cJSON_IsObject(cjson)) {
         goto done;
     }
@@ -166,6 +166,37 @@ done:
     return result;
 }
 
+int aws_json_const_iterate_object(
+    const struct aws_json_value *object,
+    aws_json_on_member_encountered_const_fn *on_member,
+    void *user_data) {
+    int result = AWS_OP_ERR;
+
+    const struct cJSON *cjson = (const struct cJSON *)object;
+    if (!cJSON_IsObject(cjson)) {
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        goto done;
+    }
+
+    const cJSON *key = NULL;
+    cJSON_ArrayForEach(key, cjson) {
+        bool should_continue = true;
+        struct aws_byte_cursor key_cur = aws_byte_cursor_from_c_str(key->string);
+        if (on_member(&key_cur, (const struct aws_json_value *)key, &should_continue, user_data)) {
+            goto done;
+        }
+
+        if (!should_continue) {
+            break;
+        }
+    }
+
+    result = AWS_OP_SUCCESS;
+
+done:
+    return result;
+}
+
 int aws_json_value_add_array_element(struct aws_json_value *array, const struct aws_json_value *value) {
 
     struct cJSON *cjson = (struct cJSON *)array;
@@ -183,8 +214,7 @@ int aws_json_value_add_array_element(struct aws_json_value *array, const struct
 }
 
 struct aws_json_value *aws_json_get_array_element(const struct aws_json_value *array, size_t index) {
-
-    struct cJSON *cjson = (struct cJSON *)array;
+    const struct cJSON *cjson = (const struct cJSON *)array;
     if (!cJSON_IsArray(cjson)) {
         aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
         return NULL;
@@ -199,7 +229,7 @@ struct aws_json_value *aws_json_get_array_element(const struct aws_json_value *a
 }
 
 size_t aws_json_get_array_size(const struct aws_json_value *array) {
-    struct cJSON *cjson = (struct cJSON *)array;
+    const struct cJSON *cjson = (const struct cJSON *)array;
     if (!cJSON_IsArray(cjson)) {
         aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
         return 0;
@@ -222,8 +252,61 @@ int aws_json_value_remove_array_element(struct aws_json_value *array, size_t ind
     return AWS_OP_SUCCESS;
 }
 
+int aws_json_const_iterate_array(
+    const struct aws_json_value *array,
+    aws_json_on_value_encountered_const_fn *on_value,
+    void *user_data) {
+    int result = AWS_OP_ERR;
+
+    const struct cJSON *cjson = (const struct cJSON *)array;
+    if (!cJSON_IsArray(cjson)) {
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        goto done;
+    }
+
+    size_t idx = 0;
+    const cJSON *value = NULL;
+    cJSON_ArrayForEach(value, cjson) {
+        bool should_continue = true;
+        if (on_value(idx, (const struct aws_json_value *)value, &should_continue, user_data)) {
+            goto done;
+        }
+
+        if (!should_continue) {
+            break;
+        }
+        ++idx;
+    }
+
+    result = AWS_OP_SUCCESS;
+
+done:
+    return result;
+}
+
+bool aws_json_value_compare(const struct aws_json_value *a, const struct aws_json_value *b, bool is_case_sensitive) {
+    const struct cJSON *cjson_a = (const struct cJSON *)a;
+    const struct cJSON *cjson_b = (const struct cJSON *)b;
+    return cJSON_Compare(cjson_a, cjson_b, is_case_sensitive);
+}
+
+struct aws_json_value *aws_json_value_duplicate(const struct aws_json_value *value) {
+    const struct cJSON *cjson = (const struct cJSON *)value;
+    if (cJSON_IsInvalid(cjson)) {
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
+    struct cJSON *ret = cJSON_Duplicate(cjson, true);
+    if (ret == NULL) {
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    return (void *)ret;
+}
+
 bool aws_json_value_is_string(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -231,7 +314,7 @@ bool aws_json_value_is_string(const struct aws_json_value *value) {
 }
 
 bool aws_json_value_is_number(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -239,7 +322,7 @@ bool aws_json_value_is_number(const struct aws_json_value *value) {
 }
 
 bool aws_json_value_is_array(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -247,7 +330,7 @@ bool aws_json_value_is_array(const struct aws_json_value *value) {
 }
 
 bool aws_json_value_is_boolean(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -255,7 +338,7 @@ bool aws_json_value_is_boolean(const struct aws_json_value *value) {
 }
 
 bool aws_json_value_is_null(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -263,7 +346,7 @@ bool aws_json_value_is_null(const struct aws_json_value *value) {
 }
 
 bool aws_json_value_is_object(const struct aws_json_value *value) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return false;
     }
@@ -299,20 +382,22 @@ void aws_json_module_cleanup(void) {
 
 void aws_json_value_destroy(struct aws_json_value *value) {
     struct cJSON *cjson = (struct cJSON *)value;
-    if (!cJSON_IsInvalid(cjson)) {
+    /* Note: cJSON_IsInvalid returns false for NULL values, so we need explicit
+        check for NULL to skip delete */
+    if (cjson != NULL && !cJSON_IsInvalid(cjson)) {
         cJSON_Delete(cjson);
     }
 }
 
 int aws_byte_buf_append_json_string(const struct aws_json_value *value, struct aws_byte_buf *output) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
     char *tmp = cJSON_PrintUnformatted(cjson);
     if (tmp == NULL) {
-        return AWS_OP_ERR;
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
     // Append the text to the byte buffer
@@ -323,14 +408,14 @@ int aws_byte_buf_append_json_string(const struct aws_json_value *value, struct a
 }
 
 int aws_byte_buf_append_json_string_formatted(const struct aws_json_value *value, struct aws_byte_buf *output) {
-    struct cJSON *cjson = (struct cJSON *)value;
+    const struct cJSON *cjson = (const struct cJSON *)value;
     if (cJSON_IsInvalid(cjson)) {
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
     char *tmp = cJSON_Print(cjson);
     if (tmp == NULL) {
-        return AWS_OP_ERR;
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
     // Append the text to the byte buffer
@@ -341,7 +426,7 @@ int aws_byte_buf_append_json_string_formatted(const struct aws_json_value *value
 }
 
 struct aws_json_value *aws_json_value_new_from_string(struct aws_allocator *allocator, struct aws_byte_cursor string) {
-    struct aws_string *tmp = aws_string_new_from_cursor((struct aws_allocator *)allocator, &string);
+    struct aws_string *tmp = aws_string_new_from_cursor(allocator, &string);
     struct cJSON *cjson = cJSON_Parse(aws_string_c_str(tmp));
     aws_string_destroy_secure(tmp);
     return (void *)cjson;
