@@ -63,32 +63,36 @@ static int s_test_memtrace_count(struct aws_allocator *allocator, void *ctx) {
 }
 AWS_TEST_CASE(test_memtrace_count, s_test_memtrace_count)
 
-#if defined(__GNUC__) || defined(__clang__)
-#    define AWS_PREVENT_OPTIMIZATION __asm__ __volatile__("" ::: "memory")
-#else
-#    define AWS_PREVENT_OPTIMIZATION
+/* Disable optimization for tests to make sure those allocation function still exists. */
+#if defined(__clang__)
+#    pragma clang optimize off
+#elif defined(__GNUC__) || defined(__GNUG__)
+#    pragma GCC push_options
+#    pragma GCC optimize("O0")
 #endif
 
 AWS_NO_INLINE void *s_alloc_1(struct aws_allocator *allocator, size_t size) {
-    AWS_PREVENT_OPTIMIZATION;
     return aws_mem_acquire(allocator, size);
 }
 
 AWS_NO_INLINE void *s_alloc_2(struct aws_allocator *allocator, size_t size) {
-    AWS_PREVENT_OPTIMIZATION;
     return aws_mem_acquire(allocator, size);
 }
 
 AWS_NO_INLINE void *s_alloc_3(struct aws_allocator *allocator, size_t size) {
-    AWS_PREVENT_OPTIMIZATION;
     return aws_mem_acquire(allocator, size);
 }
 
 AWS_NO_INLINE void *s_alloc_4(struct aws_allocator *allocator, size_t size) {
-    AWS_PREVENT_OPTIMIZATION;
     return aws_mem_acquire(allocator, size);
 }
 
+#if defined(__clang__)
+#    pragma clang optimize on
+#elif defined(__GNUC__) || defined(__GNUG__)
+#    pragma GCC pop_options
+#endif
+
 static struct aws_logger s_test_logger;
 
 static int s_test_memtrace_stacks(struct aws_allocator *allocator, void *ctx) {
@@ -147,10 +151,16 @@ static int s_test_memtrace_stacks(struct aws_allocator *allocator, void *ctx) {
     char s_alloc_2_addr[32];
     char s_alloc_3_addr[32];
     char s_alloc_4_addr[32];
+#    if defined(_MSC_VER)
+#        pragma warning(push)
+#        pragma warning(disable : 4054) /* type cast function pointer to data pointer */
     snprintf(s_alloc_1_addr, AWS_ARRAY_SIZE(s_alloc_1_addr), "0x%tx", (uintptr_t)(void *)s_alloc_1);
     snprintf(s_alloc_2_addr, AWS_ARRAY_SIZE(s_alloc_2_addr), "0x%tx", (uintptr_t)(void *)s_alloc_2);
     snprintf(s_alloc_3_addr, AWS_ARRAY_SIZE(s_alloc_3_addr), "0x%tx", (uintptr_t)(void *)s_alloc_3);
     snprintf(s_alloc_4_addr, AWS_ARRAY_SIZE(s_alloc_4_addr), "0x%tx", (uintptr_t)(void *)s_alloc_4);
+#        pragma warning(pop)
+#    endif /* defined(_MSC_VER) */
+
     const char *log_buffer = (const char *)test_logger->log_buffer.buffer;
     ASSERT_TRUE(strstr(log_buffer, "s_alloc_1") || strstr(log_buffer, s_alloc_1_addr));
     ASSERT_TRUE(strstr(log_buffer, "s_alloc_2") || strstr(log_buffer, s_alloc_2_addr));
