@@ -58,6 +58,8 @@ struct cm_tester_options {
     struct aws_http2_setting *initial_settings_array;
     size_t num_initial_settings;
     bool self_lib_init;
+    const struct aws_byte_cursor *verify_network_interface_names_array;
+    size_t num_network_interface_names;
 };
 
 struct cm_tester {
@@ -73,6 +75,8 @@ struct cm_tester {
     struct aws_tls_ctx_options tls_ctx_options;
     struct aws_tls_connection_options tls_connection_options;
     struct aws_http_proxy_options *verify_proxy_options;
+    const struct aws_byte_cursor *verify_network_interface_names_array;
+    size_t num_network_interface_names;
 
     struct aws_mutex lock;
     struct aws_condition_variable signal;
@@ -219,6 +223,8 @@ static int s_cm_tester_init(struct cm_tester_options *options) {
         .http2_prior_knowledge = !options->use_tls && options->http2,
         .initial_settings_array = options->initial_settings_array,
         .num_initial_settings = options->num_initial_settings,
+        .network_interface_names_array = options->verify_network_interface_names_array,
+        .num_network_interface_names = options->num_network_interface_names,
     };
 
     if (options->mock_table) {
@@ -234,6 +240,8 @@ static int s_cm_tester_init(struct cm_tester_options *options) {
     }
 
     tester->mock_table = options->mock_table;
+    tester->verify_network_interface_names_array = options->verify_network_interface_names_array;
+    tester->num_network_interface_names = options->num_network_interface_names;
 
     aws_atomic_store_int(&tester->next_connection_id, 0);
 
@@ -729,6 +737,13 @@ static int s_aws_http_connection_manager_create_connection_sync_mock(
     const struct aws_http_client_connection_options *options) {
     struct cm_tester *tester = &s_tester;
 
+    if (tester->num_network_interface_names) {
+        struct aws_byte_cursor interface_name =
+            tester->verify_network_interface_names_array
+                [aws_atomic_load_int(&tester->next_connection_id) % tester->num_network_interface_names];
+        ASSERT_TRUE(aws_byte_cursor_eq_c_str(&interface_name, options->socket_options->network_interface_name));
+    }
+
     size_t next_connection_id = aws_atomic_fetch_add(&tester->next_connection_id, 1);
 
     ASSERT_SUCCESS(aws_mutex_lock(&tester->lock));
@@ -819,6 +834,39 @@ static struct aws_http_connection_manager_system_vtable s_synchronous_mocks = {
     .aws_http_connection_get_version = s_aws_http_connection_manager_connection_get_version_sync_mock,
 };
 
+static int s_test_connection_manager_with_network_interface_list(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    struct aws_byte_cursor *interface_names_array = aws_mem_calloc(allocator, 3, sizeof(struct aws_byte_cursor));
+    interface_names_array[0] = aws_byte_cursor_from_c_str("ens32");
+    interface_names_array[1] = aws_byte_cursor_from_c_str("ens64");
+    interface_names_array[2] = aws_byte_cursor_from_c_str("ens96");
+
+    struct cm_tester_options options = {
+        .allocator = allocator,
+        .max_connections = 20,
+        .mock_table = &s_synchronous_mocks,
+        .verify_network_interface_names_array = interface_names_array,
+    };
+
+    ASSERT_SUCCESS(s_cm_tester_init(&options));
+    size_t num_connections = 6;
+    for (size_t i = 0; i < num_connections; ++i) {
+        s_add_mock_connections(1, AWS_NCRT_SUCCESS, i % 1 == 0);
+    }
+    s_acquire_connections(num_connections);
+
+    ASSERT_SUCCESS(s_wait_on_connection_reply_count(num_connections));
+    ASSERT_SUCCESS(s_release_connections(num_connections, false));
+    ASSERT_UINT_EQUALS(0, s_tester.connection_errors);
+
+    ASSERT_SUCCESS(s_cm_tester_clean_up());
+    aws_mem_release(allocator, interface_names_array);
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(
+    test_connection_manager_with_network_interface_list,
+    s_test_connection_manager_with_network_interface_list);
+
 static int s_test_connection_manager_acquire_release_mix_synchronous(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
@@ -1432,7 +1480,7 @@ static struct aws_byte_cursor s_get_proxy_host_for_test(
     return aws_byte_cursor_from_string(host_string);
 }
 
-static uint16_t s_get_proxy_port_for_test(
+static uint32_t s_get_proxy_port_for_test(
     struct proxy_integration_configurations *configs,
     enum proxy_test_type proxy_test_type,
     enum aws_http_proxy_authentication_type auth_type,
@@ -1447,7 +1495,7 @@ static uint16_t s_get_proxy_port_for_test(
     } else {
         port_string = configs->http_proxy_port;
     }
-    return (uint16_t)atoi(aws_string_c_str(port_string));
+    return (uint32_t)atoi(aws_string_c_str(port_string));
 }
 static struct aws_string *s_get_proxy_url_for_test(
     struct proxy_integration_configurations *configs,
