@@ -25,6 +25,22 @@
 
 #    include <aws/io/private/pki_utils.h>
 
+/* badssl.com has occasional lags, make this timeout longer so we have a
+ * higher chance of actually testing something. */
+#    define BADSSL_TIMEOUT_MS 10000
+
+bool s_is_badssl_being_flaky(const struct aws_string *host_name, int error_code) {
+    if (strstr(aws_string_c_str(host_name), "badssl.com") != NULL) {
+        if (error_code == AWS_IO_SOCKET_TIMEOUT || error_code == AWS_IO_TLS_NEGOTIATION_TIMEOUT) {
+            fprintf(
+                AWS_TESTING_REPORT_FD,
+                "Warning: badssl.com is timing out right now. Maybe run the test again later?\n");
+            return true;
+        }
+    }
+    return false;
+}
+
 struct tls_test_args {
     struct aws_allocator *allocator;
     struct aws_mutex *mutex;
@@ -672,7 +688,7 @@ struct default_host_callback_data {
 static int s_verify_negotiation_fails_helper(
     struct aws_allocator *allocator,
     const struct aws_string *host_name,
-    uint16_t port,
+    uint32_t port,
     struct aws_tls_ctx_options *client_ctx_options) {
     struct aws_tls_ctx *client_ctx = aws_tls_client_ctx_new(allocator, client_ctx_options);
 
@@ -699,9 +715,7 @@ static int s_verify_negotiation_fails_helper(
 
     struct aws_socket_options options;
     AWS_ZERO_STRUCT(options);
-    /* badssl.com is great but has occasional lags, make this timeout longer so we have a
-       higher chance of actually testing something. */
-    options.connect_timeout_ms = 10000;
+    options.connect_timeout_ms = BADSSL_TIMEOUT_MS;
     options.type = AWS_SOCKET_STREAM;
     options.domain = AWS_SOCKET_IPV4;
 
@@ -735,16 +749,12 @@ static int s_verify_negotiation_fails_helper(
 
     ASSERT_TRUE(outgoing_args.error_invoked);
 
-    /* we're talking to an external internet endpoint, yeah this sucks... we don't know for sure that
-       this failed for the right reasons, but there's not much we can do about it.*/
-    if (outgoing_args.last_error_code != AWS_IO_SOCKET_TIMEOUT) {
-        ASSERT_INT_EQUALS(AWS_IO_TLS_ERROR_NEGOTIATION_FAILURE, outgoing_args.last_error_code);
-    } else {
-        fprintf(
-            stderr,
-            "Warning: the connection timed out and we're not completely certain"
-            " that this fails for the right reasons. Maybe run the test again?\n");
+    if (s_is_badssl_being_flaky(host_name, outgoing_args.last_error_code)) {
+        return AWS_OP_SKIP;
     }
+
+    ASSERT_INT_EQUALS(AWS_IO_TLS_ERROR_NEGOTIATION_FAILURE, outgoing_args.last_error_code);
+
     aws_client_bootstrap_release(client_bootstrap);
 
     aws_tls_ctx_release(client_ctx);
@@ -755,7 +765,7 @@ static int s_verify_negotiation_fails_helper(
 static int s_verify_negotiation_fails(
     struct aws_allocator *allocator,
     const struct aws_string *host_name,
-    uint16_t port,
+    uint32_t port,
     void (*context_options_override_fn)(struct aws_tls_ctx_options *)) {
 
     aws_io_library_init(allocator);
@@ -1019,7 +1029,7 @@ static int s_tls_client_channel_negotiation_error_socket_closed_fn(struct aws_al
     (void)ctx;
 
     const char *host_name = "aws-crt-test-stuff.s3.amazonaws.com";
-    uint16_t port = 80; /* Note: intentionally wrong and not 443 */
+    uint32_t port = 80; /* Note: intentionally wrong and not 443 */
 
     aws_io_library_init(allocator);
 
@@ -1084,7 +1094,7 @@ AWS_TEST_CASE(
 static int s_verify_good_host(
     struct aws_allocator *allocator,
     const struct aws_string *host_name,
-    uint16_t port,
+    uint32_t port,
     void (*override_tls_options_fn)(struct aws_tls_ctx_options *)) {
 
     aws_io_library_init(allocator);
@@ -1126,7 +1136,7 @@ static int s_verify_good_host(
 
     struct aws_socket_options options;
     AWS_ZERO_STRUCT(options);
-    options.connect_timeout_ms = 10000;
+    options.connect_timeout_ms = BADSSL_TIMEOUT_MS;
     options.type = AWS_SOCKET_STREAM;
     options.domain = AWS_SOCKET_IPV4;
 
@@ -1159,6 +1169,10 @@ static int s_verify_good_host(
         &c_tester.condition_variable, &c_tester.mutex, s_tls_channel_setup_predicate, &outgoing_args));
     ASSERT_SUCCESS(aws_mutex_unlock(&c_tester.mutex));
 
+    if (s_is_badssl_being_flaky(host_name, outgoing_args.last_error_code)) {
+        return AWS_OP_SKIP;
+    }
+
     ASSERT_FALSE(outgoing_args.error_invoked);
     struct aws_byte_buf expected_protocol = aws_byte_buf_from_c_str("http/1.1");
     /* check ALPN and SNI was properly negotiated */
@@ -2124,14 +2138,9 @@ static void s_import_cert(void *ctx) {
 #    endif /* !AWS_OS_IOS */
 }
 
-#    define NUM_PAIRS 1
+#    define NUM_PAIRS 2
 static int s_test_concurrent_cert_import(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
-    /* temporarily disable this on apple until we can fix importing to be more robust */
-    /* temporarily disable this on linux until we can make CRYPTO_zalloc behave and stop angering ASan */
-#    if defined(__APPLE__) || defined(__linux__)
-    return AWS_OP_SUCCESS;
-#    endif
 
     aws_io_library_init(allocator);
 
@@ -2178,6 +2187,45 @@ static int s_test_concurrent_cert_import(struct aws_allocator *allocator, void *
 
 AWS_TEST_CASE(test_concurrent_cert_import, s_test_concurrent_cert_import)
 
+static int s_test_duplicate_cert_import(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    aws_io_library_init(allocator);
+    struct aws_byte_buf cert_buf = {0};
+    struct aws_byte_buf key_buf = {0};
+
+#    if !defined(AWS_OS_IOS)
+
+    ASSERT_SUCCESS(aws_byte_buf_init_from_file(&cert_buf, allocator, "testcert0.pem"));
+    ASSERT_SUCCESS(aws_byte_buf_init_from_file(&key_buf, allocator, "testkey.pem"));
+    struct aws_byte_cursor cert_cur = aws_byte_cursor_from_buf(&cert_buf);
+    struct aws_byte_cursor key_cur = aws_byte_cursor_from_buf(&key_buf);
+    struct aws_tls_ctx_options tls_options = {0};
+    AWS_FATAL_ASSERT(
+        AWS_OP_SUCCESS == aws_tls_ctx_options_init_client_mtls(&tls_options, allocator, &cert_cur, &key_cur));
+
+    /* import happens in here */
+    struct aws_tls_ctx *tls = aws_tls_client_ctx_new(allocator, &tls_options);
+    AWS_FATAL_ASSERT(tls);
+    aws_tls_ctx_release(tls);
+    /* import the same certs twice */
+    tls = aws_tls_client_ctx_new(allocator, &tls_options);
+    AWS_FATAL_ASSERT(tls);
+    aws_tls_ctx_release(tls);
+
+    aws_tls_ctx_options_clean_up(&tls_options);
+#    endif /* !AWS_OS_IOS */
+
+    /* clean up */
+    aws_byte_buf_clean_up(&cert_buf);
+    aws_byte_buf_clean_up(&key_buf);
+    aws_io_library_clean_up();
+
+    return AWS_OP_SUCCESS;
+}
+
+AWS_TEST_CASE(test_duplicate_cert_import, s_test_duplicate_cert_import)
+
 static int s_tls_destroy_null_context(struct aws_allocator *allocator, void *ctx) {
     (void)allocator;
     (void)ctx;
