@@ -59,7 +59,7 @@ struct aws_s3_meta_request_event {
     enum aws_s3_meta_request_event_type {
         AWS_S3_META_REQUEST_EVENT_RESPONSE_BODY, /* body_callback */
         AWS_S3_META_REQUEST_EVENT_PROGRESS,      /* progress_callback */
-        /* TODO: AWS_S3_META_REQUEST_EVENT_TELEMETRY */
+        AWS_S3_META_REQUEST_EVENT_TELEMETRY,     /* telemetry_callback */
     } type;
 
     union {
@@ -72,6 +72,11 @@ struct aws_s3_meta_request_event {
         struct {
             struct aws_s3_meta_request_progress info;
         } progress;
+
+        /* data for AWS_S3_META_REQUEST_EVENT_TELEMETRY */
+        struct {
+            struct aws_s3_request_metrics *metrics;
+        } telemetry;
     } u;
 };
 
@@ -138,9 +143,11 @@ struct aws_s3_meta_request {
     /* The meta request's outgoing body comes from one of these:
      * 1) request_body_async_stream: if set, then async stream 1 part at a time
      * 2) request_body_parallel_stream: if set, then stream multiple parts in parallel
-     * 3) initial_request_message's body_stream: else synchronously stream parts */
+     * 3) request_body_using_async_writes: if set, then synchronously copy async_write data from 1 part at a time
+     * 4) initial_request_message's body_stream: else synchronously stream parts */
     struct aws_async_input_stream *request_body_async_stream;
     struct aws_parallel_input_stream *request_body_parallel_stream;
+    bool request_body_using_async_writes;
 
     /* Part size to use for uploads and downloads.  Passed down by the creating client. */
     const size_t part_size;
@@ -221,6 +228,30 @@ struct aws_s3_meta_request {
         /* True if the finish result has been set. */
         uint32_t finish_result_set : 1;
 
+        /* To track aws_s3_requests with cancellable HTTP streams */
+        struct aws_linked_list cancellable_http_streams_list;
+
+        /* Data for async-writes. */
+        struct {
+            /* Whether a part request can be sent (we have 1 part's worth of data, or EOF) */
+            bool ready_to_send;
+
+            /* True once user passes `eof` to their final write() call */
+            bool eof;
+
+            /* Holds buffered data we can't immediately send.
+             * The length will always be less than part-size */
+            struct aws_byte_buf buffered_data;
+            struct aws_s3_buffer_pool_ticket *buffered_data_ticket;
+
+            /* Waker callback.
+             * Stored if a poll_write() call returns result.is_pending
+             * because we already had 1 part's worth of data.
+             * Invoked when we're ready to accept another poll_write() call. */
+            aws_simple_completion_callback *waker;
+            void *waker_user_data;
+        } async_write;
+
     } synced_data;
 
     /* Anything in this structure should only ever be accessed by the client on its process work event loop task. */
@@ -359,6 +390,9 @@ void aws_s3_meta_request_add_event_for_delivery_synced(
  * The meta-request's finish callback must not be invoked until this returns false. */
 bool aws_s3_meta_request_are_events_out_for_delivery_synced(struct aws_s3_meta_request *meta_request);
 
+/* Cancel the requests with cancellable HTTP stream for the meta request */
+void aws_s3_meta_request_cancel_cancellable_requests_synced(struct aws_s3_meta_request *meta_request, int error_code);
+
 /* Asynchronously read from the meta request's input stream. Should always be done outside of any mutex,
  * as reading from the stream could cause user code to call back into aws-c-s3.
  * This will fill the buffer to capacity, unless end of stream is reached.
@@ -374,8 +408,6 @@ struct aws_future_bool *aws_s3_meta_request_read_body(
     uint64_t offset,
     struct aws_byte_buf *buffer);
 
-bool aws_s3_meta_request_body_has_no_more_data(const struct aws_s3_meta_request *meta_request);
-
 /* Set the meta request finish result as failed. This is meant to be called sometime before aws_s3_meta_request_finish.
  * Subsequent calls to this function or to aws_s3_meta_request_set_success_synced will not overwrite the end result of
  * the meta request. */
