@@ -21,7 +21,10 @@
     static int s_test_##NAME(struct aws_allocator *allocator, void *ctx)
 
 #define DEFINE_HEADER(NAME, VALUE)                                                                                     \
-    { .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(NAME), .value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(VALUE), }
+    {                                                                                                                  \
+        .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(NAME),                                                           \
+        .value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(VALUE),                                                         \
+    }
 
 struct aws_s3express_client_tester {
     struct aws_allocator *allocator;
@@ -228,8 +231,6 @@ static int s_s3express_client_put_test_helper(struct aws_allocator *allocator, s
 
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
 
-    char endpoint[] = "crts-east1--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com";
-    struct aws_byte_cursor host_cursor = aws_byte_cursor_from_c_str(endpoint);
     struct aws_byte_cursor key_cursor = aws_byte_cursor_from_c_str("/crt-test");
 
     struct aws_s3_client_config client_config = {
@@ -243,13 +244,23 @@ static int s_s3express_client_put_test_helper(struct aws_allocator *allocator, s
     struct aws_s3_client *client = aws_s3_client_new(allocator, &client_config);
 
     ASSERT_SUCCESS(s_s3express_put_object_request(
-        allocator, client, content_length, &tester, host_cursor, key_cursor, region_cursor));
+        allocator,
+        client,
+        content_length,
+        &tester,
+        g_test_s3express_bucket_use1_az4_endpoint,
+        key_cursor,
+        region_cursor));
 
     struct aws_byte_cursor west2_region_cursor = aws_byte_cursor_from_c_str("us-west-2");
-    char west2_endpoint[] = "crts-west2--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com";
-    struct aws_byte_cursor west2_host_cursor = aws_byte_cursor_from_c_str(west2_endpoint);
     ASSERT_SUCCESS(s_s3express_put_object_request(
-        allocator, client, content_length, &tester, west2_host_cursor, key_cursor, west2_region_cursor));
+        allocator,
+        client,
+        content_length,
+        &tester,
+        g_test_s3express_bucket_usw2_az1_endpoint,
+        key_cursor,
+        west2_region_cursor));
 
     aws_s3_client_release(client);
     aws_s3_tester_clean_up(&tester);
@@ -280,13 +291,9 @@ TEST_CASE(s3express_client_put_object_multipart_multiple) {
 
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
 
-    char endpoint[] = "crts-east1--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com";
-    struct aws_byte_cursor host_cursor = aws_byte_cursor_from_c_str(endpoint);
     struct aws_byte_cursor key_cursor = aws_byte_cursor_from_c_str("/crt-test");
 
     struct aws_byte_cursor west2_region_cursor = aws_byte_cursor_from_c_str("us-west-2");
-    char west2_endpoint[] = "crts-west2--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com";
-    struct aws_byte_cursor west2_host_cursor = aws_byte_cursor_from_c_str(west2_endpoint);
 
     struct aws_s3_client_config client_config = {
         .part_size = MB_TO_BYTES(5),
@@ -302,11 +309,11 @@ TEST_CASE(s3express_client_put_object_multipart_multiple) {
         input_streams[i] = aws_s3_test_input_stream_new(allocator, MB_TO_BYTES(10));
 
         struct aws_byte_cursor request_region = region_cursor;
-        struct aws_byte_cursor request_host = host_cursor;
+        struct aws_byte_cursor request_host = g_test_s3express_bucket_use1_az4_endpoint;
         if (i % 2 == 0) {
             /* Make half of request to east1 and rest half to west2 */
             request_region = west2_region_cursor;
-            request_host = west2_host_cursor;
+            request_host = g_test_s3express_bucket_usw2_az1_endpoint;
         }
 
         struct aws_http_message *message = aws_s3_test_put_object_request_new(
@@ -401,8 +408,6 @@ TEST_CASE(s3express_client_put_object_long_running_session_refresh) {
 
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
 
-    char endpoint[] = "crts-east1--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com";
-    struct aws_byte_cursor host_cursor = aws_byte_cursor_from_c_str(endpoint);
     struct aws_byte_cursor key_cursor = aws_byte_cursor_from_c_str("/crt-test");
 
     struct aws_s3_client_config client_config = {
@@ -421,7 +426,12 @@ TEST_CASE(s3express_client_put_object_long_running_session_refresh) {
         struct aws_input_stream *upload_stream = aws_s3_test_input_stream_new(allocator, MB_TO_BYTES(10));
 
         struct aws_http_message *message = aws_s3_test_put_object_request_new(
-            allocator, &host_cursor, key_cursor, g_test_body_content_type, upload_stream, 0);
+            allocator,
+            &g_test_s3express_bucket_use1_az4_endpoint,
+            key_cursor,
+            g_test_body_content_type,
+            upload_stream,
+            0);
         struct aws_s3_meta_request_options options;
         AWS_ZERO_STRUCT(options);
         options.type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT;
@@ -473,10 +483,6 @@ TEST_CASE(s3express_client_get_object) {
 
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
 
-    char endpoint[] = "crts-east1--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com";
-    struct aws_byte_cursor host_cursor = aws_byte_cursor_from_c_str(endpoint);
-    struct aws_byte_cursor key_cursor = aws_byte_cursor_from_c_str("/crt-download-10MB");
-
     struct aws_s3_client_config client_config = {
         .part_size = MB_TO_BYTES(5),
         .enable_s3express = true,
@@ -487,7 +493,8 @@ TEST_CASE(s3express_client_get_object) {
 
     struct aws_s3_client *client = aws_s3_client_new(allocator, &client_config);
 
-    struct aws_http_message *message = aws_s3_test_get_object_request_new(allocator, host_cursor, key_cursor);
+    struct aws_http_message *message = aws_s3_test_get_object_request_new(
+        allocator, g_test_s3express_bucket_use1_az4_endpoint, g_pre_existing_object_10MB);
 
     struct aws_s3_meta_request_options options;
     AWS_ZERO_STRUCT(options);
@@ -532,10 +539,6 @@ TEST_CASE(s3express_client_get_object_multiple) {
 
     struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
 
-    char endpoint[] = "crts-east1--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com";
-    struct aws_byte_cursor host_cursor = aws_byte_cursor_from_c_str(endpoint);
-    struct aws_byte_cursor key_cursor = aws_byte_cursor_from_c_str("/crt-download-10MB");
-
     struct aws_s3_client_config client_config = {
         .part_size = MB_TO_BYTES(5),
         .enable_s3express = true,
@@ -548,7 +551,8 @@ TEST_CASE(s3express_client_get_object_multiple) {
 
     for (size_t i = 0; i < num_meta_requests; ++i) {
 
-        struct aws_http_message *message = aws_s3_test_get_object_request_new(allocator, host_cursor, key_cursor);
+        struct aws_http_message *message = aws_s3_test_get_object_request_new(
+            allocator, g_test_s3express_bucket_use1_az4_endpoint, g_pre_existing_object_10MB);
 
         struct aws_s3_meta_request_options options;
         AWS_ZERO_STRUCT(options);
@@ -589,3 +593,58 @@ TEST_CASE(s3express_client_get_object_multiple) {
     aws_s3_tester_clean_up(&tester);
     return AWS_OP_SUCCESS;
 }
+
+TEST_CASE(s3express_client_get_object_create_session_error) {
+    (void)ctx;
+
+    struct aws_s3_tester tester;
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
+
+    struct aws_byte_cursor region_cursor = aws_byte_cursor_from_c_str("us-east-1");
+
+    struct aws_s3_client_config client_config = {
+        .part_size = MB_TO_BYTES(5),
+        .enable_s3express = true,
+        .region = region_cursor,
+    };
+
+    ASSERT_SUCCESS(aws_s3_tester_bind_client(&tester, &client_config, AWS_S3_TESTER_BIND_CLIENT_SIGNING));
+
+    struct aws_s3_client *client = aws_s3_client_new(allocator, &client_config);
+
+    struct aws_byte_cursor my_dummy_endpoint = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
+        "non-exist-bucket-test--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com");
+
+    struct aws_http_message *message =
+        aws_s3_test_get_object_request_new(allocator, my_dummy_endpoint, g_pre_existing_object_10MB);
+
+    struct aws_s3_meta_request_options options;
+    AWS_ZERO_STRUCT(options);
+    options.type = AWS_S3_META_REQUEST_TYPE_GET_OBJECT;
+    options.message = message;
+    struct aws_signing_config_aws s3express_signing_config = {
+        .algorithm = AWS_SIGNING_ALGORITHM_V4_S3EXPRESS,
+        .service = g_s3express_service_name,
+    };
+    options.signing_config = &s3express_signing_config;
+    struct aws_s3_meta_request_test_results meta_request_test_results;
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
+
+    ASSERT_SUCCESS(aws_s3_tester_bind_meta_request(&tester, &options, &meta_request_test_results));
+
+    struct aws_s3_meta_request *meta_request = aws_s3_client_make_meta_request(client, &options);
+    ASSERT_TRUE(meta_request != NULL);
+    /* Wait for the request to finish. */
+    aws_s3_tester_wait_for_meta_request_finish(&tester);
+    ASSERT_UINT_EQUALS(meta_request_test_results.finished_error_code, AWS_ERROR_S3EXPRESS_CREATE_SESSION_FAILED);
+
+    meta_request = aws_s3_meta_request_release(meta_request);
+    aws_s3_tester_wait_for_meta_request_shutdown(&tester);
+
+    aws_s3_meta_request_test_results_clean_up(&meta_request_test_results);
+
+    aws_http_message_release(message);
+    aws_s3_client_release(client);
+    aws_s3_tester_clean_up(&tester);
+    return AWS_OP_SUCCESS;
+}
