@@ -55,6 +55,8 @@ const struct aws_byte_cursor g_pre_existing_object_kms_10MB =
 const struct aws_byte_cursor g_pre_existing_object_aes256_10MB =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/pre-existing-10MB-aes256");
 const struct aws_byte_cursor g_pre_existing_empty_object = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/pre-existing-empty");
+const struct aws_byte_cursor g_pre_existing_object_async_error_xml =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/pre-existing-async-error-xml");
 
 const struct aws_byte_cursor g_put_object_prefix = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/upload/put-object-test");
 const struct aws_byte_cursor g_upload_folder = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/upload");
@@ -440,7 +442,9 @@ int aws_s3_tester_init(struct aws_allocator *allocator, struct aws_s3_tester *te
             &tester->default_signing_config, g_test_s3_region, tester->credentials_provider);
     }
 #else
-    { tester->default_signing_config = tester->anonymous_signing_config; }
+    {
+        tester->default_signing_config = tester->anonymous_signing_config;
+    }
 #endif
 
     return AWS_OP_SUCCESS;
@@ -1344,6 +1348,10 @@ int aws_s3_tester_client_new(
     if (options->use_proxy) {
         client_config.proxy_options = &proxy_options;
     }
+    if (options->num_network_interface_names) {
+        client_config.network_interface_names_array = options->network_interface_names_array;
+        client_config.num_network_interface_names = options->num_network_interface_names;
+    }
 
     struct aws_tls_connection_options tls_connection_options;
     AWS_ZERO_STRUCT(tls_connection_options);
@@ -1682,9 +1690,11 @@ int aws_s3_tester_send_meta_request_with_options(
         if (options->meta_request_type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT) {
             /* Figure out how much is being uploaded from pre-existing message */
             struct aws_input_stream *mystery_stream = aws_http_message_get_body_stream(meta_request_options.message);
+            int64_t mystery_stream_length = 0;
             if (mystery_stream != NULL) {
-                ASSERT_SUCCESS(aws_input_stream_get_length(mystery_stream, (int64_t *)&upload_size_bytes));
+                ASSERT_SUCCESS(aws_input_stream_get_length(mystery_stream, (int64_t *)&mystery_stream_length));
             }
+            upload_size_bytes = (size_t)mystery_stream_length;
         }
     }
 
