@@ -130,6 +130,8 @@ struct aws_s3_tester_client_options {
     enum aws_s3_client_tls_usage tls_usage;
     uint64_t part_size;
     size_t max_part_size;
+    const struct aws_byte_cursor *network_interface_names_array;
+    size_t num_network_interface_names;
     uint32_t setup_region : 1;
     uint32_t use_proxy : 1;
 };
@@ -486,6 +488,7 @@ extern const struct aws_byte_cursor g_pre_existing_object_1MB;
 extern const struct aws_byte_cursor g_pre_existing_object_10MB;
 extern const struct aws_byte_cursor g_pre_existing_object_kms_10MB;
 extern const struct aws_byte_cursor g_pre_existing_object_aes256_10MB;
+extern const struct aws_byte_cursor g_pre_existing_object_async_error_xml;
 extern const struct aws_byte_cursor g_pre_existing_empty_object;
 
 extern const struct aws_byte_cursor g_put_object_prefix;
