@@ -119,8 +119,9 @@
 #include "internal.h"
 #include "rsaz_exp.h"
 
-#if !defined(OPENSSL_NO_ASM) &&                                                \
-    (defined(OPENSSL_LINUX) || defined(OPENSSL_APPLE)) &&                      \
+#if !defined(OPENSSL_NO_ASM) &&                          \
+    (defined(OPENSSL_LINUX) || defined(OPENSSL_APPLE) || \
+     defined(OPENSSL_OPENBSD)) &&                        \
     defined(OPENSSL_AARCH64)
 
 #include "../../../third_party/s2n-bignum/include/s2n-bignum_aws-lc.h"
