@@ -14,48 +14,21 @@
 #include "../cpucap/internal.h"
 #include "../delocate.h"
 #include "internal.h"
+#include "ec_nistp.h"
 
 #if !defined(OPENSSL_SMALL)
 
-// We have two implementations of the field arithmetic for P-384 curve:
-//   - Fiat-crypto
-//   - s2n-bignum
-// Both Fiat-crypto and s2n-bignum implementations are formally verified.
-// Fiat-crypto implementation is fully portable C code, while s2n-bignum
-// implements the operations in assembly for x86_64 and aarch64 platforms.
-// All the P-384 field operations supported by Fiat-crypto are supported
-// by s2n-bignum as well, so s2n-bignum can be used as a drop-in replacement
-// when appropriate. To do that we define macros for the functions.
-// For example, field addition macro is either defined as
-//   #define p384_felem_add(out, in0, in1) fiat_p384_add(out, in0, in1)
-// when Fiat-crypto is used, or as:
-//   #define p384_felem_add(out, in0, in1) bignum_add_p384(out, in0, in1)
-// when s2n-bignum is used.
-//
-// If (1) x86_64 or aarch64, (2) linux or apple, and (3) OPENSSL_NO_ASM is not
-// set, s2n-bignum path is capable.
-#if !defined(OPENSSL_NO_ASM) &&                                                \
-    (defined(OPENSSL_LINUX) || defined(OPENSSL_APPLE)) &&                      \
-    ((defined(OPENSSL_X86_64) && !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)) || \
-     defined(OPENSSL_AARCH64))
-
+#if defined(EC_NISTP_USE_S2N_BIGNUM)
 #  include "../../../third_party/s2n-bignum/include/s2n-bignum_aws-lc.h"
-
-#  define P384_USE_S2N_BIGNUM_FIELD_ARITH 1
-#  define P384_USE_64BIT_LIMBS_FELEM 1
-
 #else
-
-#  if defined(BORINGSSL_HAS_UINT128)
+#  if defined(EC_NISTP_USE_64BIT_LIMB)
 #    include "../../../third_party/fiat/p384_64.h"
-#    define P384_USE_64BIT_LIMBS_FELEM 1
 #  else
 #    include "../../../third_party/fiat/p384_32.h"
 #  endif
-
 #endif
 
-#if defined(P384_USE_64BIT_LIMBS_FELEM)
+#if defined(EC_NISTP_USE_64BIT_LIMB)
 
 #define P384_NLIMBS (6)
 typedef uint64_t p384_limb_t;
@@ -73,56 +46,23 @@ static const p384_felem p384_felem_one = {
 
 #endif  // 64BIT
 
-
-#if defined(P384_USE_S2N_BIGNUM_FIELD_ARITH)
-
-#if defined(OPENSSL_X86_64)
-// On x86_64 platforms s2n-bignum uses bmi2 and adx instruction sets
-// for some of the functions. These instructions are not supported by
-// every x86 CPU so we have to check if they are available and in case
-// they are not we fallback to slightly slower but generic implementation.
-static inline uint8_t p384_use_s2n_bignum_alt(void) {
-  return (!CRYPTO_is_BMI2_capable() || !CRYPTO_is_ADX_capable());
-}
-#else
-// On aarch64 platforms s2n-bignum has two implementations of certain
-// functions -- the default one and the alternative (suffixed _alt).
-// Depending on the architecture one version is faster than the other.
-// Generally, the "_alt" functions are faster on architectures with higher
-// multiplier throughput, for example, Graviton 3, Apple's M1 and iPhone chips.
-static inline uint8_t p384_use_s2n_bignum_alt(void) {
-  return CRYPTO_is_ARMv8_wide_multiplier_capable();
-}
-#endif
+#if defined(EC_NISTP_USE_S2N_BIGNUM)
 
 #define p384_felem_add(out, in0, in1)   bignum_add_p384(out, in0, in1)
 #define p384_felem_sub(out, in0, in1)   bignum_sub_p384(out, in0, in1)
 #define p384_felem_opp(out, in0)        bignum_neg_p384(out, in0)
 #define p384_felem_to_bytes(out, in0)   bignum_tolebytes_6(out, in0)
 #define p384_felem_from_bytes(out, in0) bignum_fromlebytes_6(out, in0)
-
-// The following four functions need bmi2 and adx support.
-#define p384_felem_mul(out, in0, in1) \
-  if (p384_use_s2n_bignum_alt()) bignum_montmul_p384_alt(out, in0, in1); \
-  else bignum_montmul_p384(out, in0, in1);
-
-#define p384_felem_sqr(out, in0) \
-  if (p384_use_s2n_bignum_alt()) bignum_montsqr_p384_alt(out, in0); \
-  else bignum_montsqr_p384(out, in0);
-
-#define p384_felem_to_mont(out, in0) \
-  if (p384_use_s2n_bignum_alt()) bignum_tomont_p384_alt(out, in0); \
-  else bignum_tomont_p384(out, in0);
-
-#define p384_felem_from_mont(out, in0) \
-  if (p384_use_s2n_bignum_alt()) bignum_deamont_p384_alt(out, in0); \
-  else bignum_deamont_p384(out, in0);
+#define p384_felem_to_mont(out, in0)    bignum_tomont_p384_selector(out, in0)
+#define p384_felem_from_mont(out, in0)  bignum_deamont_p384_selector(out, in0)
+#define p384_felem_mul(out, in0, in1)   bignum_montmul_p384_selector(out, in0, in1)
+#define p384_felem_sqr(out, in0)        bignum_montsqr_p384_selector(out, in0)
 
 static p384_limb_t p384_felem_nz(const p384_limb_t in1[P384_NLIMBS]) {
   return bignum_nonzero_6(in1);
 }
 
-#else // P384_USE_S2N_BIGNUM_FIELD_ARITH
+#else // EC_NISTP_USE_S2N_BIGNUM
 
 // Fiat-crypto implementation of field arithmetic
 #define p384_felem_add(out, in0, in1)   fiat_p384_add(out, in0, in1)
@@ -141,7 +81,7 @@ static p384_limb_t p384_felem_nz(const p384_limb_t in1[P384_NLIMBS]) {
   return ret;
 }
 
-#endif // P384_USE_S2N_BIGNUM_FIELD_ARITH
+#endif // EC_NISTP_USE_S2N_BIGNUM
 
 
 static void p384_felem_copy(p384_limb_t out[P384_NLIMBS],
@@ -301,69 +241,13 @@ static void p384_inv_square(p384_felem out,
   p384_felem_sqr(out, ret);      // 2^384 - 2^128 - 2^96 + 2^32 - 2^2 = p - 3
 }
 
-// Group operations
-// ----------------
-//
-// Building on top of the field operations we have the operations on the
-// elliptic curve group itself. Points on the curve are represented in Jacobian
-// coordinates.
-//
-// p384_point_double calculates 2*(x_in, y_in, z_in)
-//
-// The method is taken from:
-//   http://hyperelliptic.org/EFD/g1p/auto-shortw-jacobian-3.html#doubling-dbl-2001-b
-//
-// Coq transcription and correctness proof:
-// <https://github.com/mit-plv/fiat-crypto/blob/79f8b5f39ed609339f0233098dee1a3c4e6b3080/src/Curves/Weierstrass/Jacobian.v#L93>
-// <https://github.com/mit-plv/fiat-crypto/blob/79f8b5f39ed609339f0233098dee1a3c4e6b3080/src/Curves/Weierstrass/Jacobian.v#L201>
-// Outputs can equal corresponding inputs, i.e., x_out == x_in is allowed;
-// while x_out == y_in is not (maybe this works, but it's not tested).
 static void p384_point_double(p384_felem x_out,
                               p384_felem y_out,
                               p384_felem z_out,
                               const p384_felem x_in,
                               const p384_felem y_in,
                               const p384_felem z_in) {
-  p384_felem delta, gamma, beta, ftmp, ftmp2, tmptmp, alpha, fourbeta;
-  // delta = z^2
-  p384_felem_sqr(delta, z_in);
-  // gamma = y^2
-  p384_felem_sqr(gamma, y_in);
-  // beta = x*gamma
-  p384_felem_mul(beta, x_in, gamma);
-
-  // alpha = 3*(x-delta)*(x+delta)
-  p384_felem_sub(ftmp, x_in, delta);
-  p384_felem_add(ftmp2, x_in, delta);
-
-  p384_felem_add(tmptmp, ftmp2, ftmp2);
-  p384_felem_add(ftmp2, ftmp2, tmptmp);
-  p384_felem_mul(alpha, ftmp, ftmp2);
-
-  // x' = alpha^2 - 8*beta
-  p384_felem_sqr(x_out, alpha);
-  p384_felem_add(fourbeta, beta, beta);
-  p384_felem_add(fourbeta, fourbeta, fourbeta);
-  p384_felem_add(tmptmp, fourbeta, fourbeta);
-  p384_felem_sub(x_out, x_out, tmptmp);
-
-  // z' = (y + z)^2 - gamma - delta
-  // The following calculation differs from that in p256.c:
-  // an add is replaced with a sub. This saves us 5 cmovznz operations
-  // when Fiat-crypto implementation of felem_add and felem_sub is used,
-  // and also a certain number of intructions when s2n-bignum is used.
-  p384_felem_add(ftmp, y_in, z_in);
-  p384_felem_sqr(z_out, ftmp);
-  p384_felem_sub(z_out, z_out, gamma);
-  p384_felem_sub(z_out, z_out, delta);
-
-  // y' = alpha*(4*beta - x') - 8*gamma^2
-  p384_felem_sub(y_out, fourbeta, x_out);
-  p384_felem_add(gamma, gamma, gamma);
-  p384_felem_sqr(gamma, gamma);
-  p384_felem_mul(y_out, alpha, y_out);
-  p384_felem_add(gamma, gamma, gamma);
-  p384_felem_sub(y_out, y_out, gamma);
+  ec_nistp_point_double(p384_methods(), x_out, y_out, z_out, x_in, y_in, z_in);
 }
 
 // p384_point_add calculates (x1, y1, z1) + (x2, y2, z2)
@@ -383,115 +267,33 @@ static void p384_point_add(p384_felem x3, p384_felem y3, p384_felem z3,
                            const p384_felem x2,
                            const p384_felem y2,
                            const p384_felem z2) {
-  p384_felem x_out, y_out, z_out;
-  p384_limb_t z1nz = p384_felem_nz(z1);
-  p384_limb_t z2nz = p384_felem_nz(z2);
-
-  // z1z1 = z1**2
-  p384_felem z1z1;
-  p384_felem_sqr(z1z1, z1);
-
-  p384_felem u1, s1, two_z1z2;
-  if (!mixed) {
-    // z2z2 = z2**2
-    p384_felem z2z2;
-    p384_felem_sqr(z2z2, z2);
-
-    // u1 = x1*z2z2
-    p384_felem_mul(u1, x1, z2z2);
-
-    // two_z1z2 = (z1 + z2)**2 - (z1z1 + z2z2) = 2z1z2
-    p384_felem_add(two_z1z2, z1, z2);
-    p384_felem_sqr(two_z1z2, two_z1z2);
-    p384_felem_sub(two_z1z2, two_z1z2, z1z1);
-    p384_felem_sub(two_z1z2, two_z1z2, z2z2);
-
-    // s1 = y1 * z2**3
-    p384_felem_mul(s1, z2, z2z2);
-    p384_felem_mul(s1, s1, y1);
-  } else {
-    // We'll assume z2 = 1 (special case z2 = 0 is handled later).
-
-    // u1 = x1*z2z2
-    p384_felem_copy(u1, x1);
-    // two_z1z2 = 2z1z2
-    p384_felem_add(two_z1z2, z1, z1);
-    // s1 = y1 * z2**3
-    p384_felem_copy(s1, y1);
-  }
-
-  // u2 = x2*z1z1
-  p384_felem u2;
-  p384_felem_mul(u2, x2, z1z1);
-
-  // h = u2 - u1
-  p384_felem h;
-  p384_felem_sub(h, u2, u1);
-
-  p384_limb_t xneq = p384_felem_nz(h);
-
-  // z_out = two_z1z2 * h
-  p384_felem_mul(z_out, h, two_z1z2);
-
-  // z1z1z1 = z1 * z1z1
-  p384_felem z1z1z1;
-  p384_felem_mul(z1z1z1, z1, z1z1);
-
-  // s2 = y2 * z1**3
-  p384_felem s2;
-  p384_felem_mul(s2, y2, z1z1z1);
-
-  // r = (s2 - s1)*2
-  p384_felem r;
-  p384_felem_sub(r, s2, s1);
-  p384_felem_add(r, r, r);
-
-  p384_limb_t yneq = p384_felem_nz(r);
-
-  // This case will never occur in the constant-time |ec_GFp_mont_mul|.
-  p384_limb_t is_nontrivial_double = constant_time_is_zero_w(xneq | yneq) &
-                                    ~constant_time_is_zero_w(z1nz) &
-                                    ~constant_time_is_zero_w(z2nz);
-  if (constant_time_declassify_w(is_nontrivial_double)) {
-    p384_point_double(x3, y3, z3, x1, y1, z1);
-    return;
-  }
-
-  // I = (2h)**2
-  p384_felem i;
-  p384_felem_add(i, h, h);
-  p384_felem_sqr(i, i);
-
-  // J = h * I
-  p384_felem j;
-  p384_felem_mul(j, h, i);
-
-  // V = U1 * I
-  p384_felem v;
-  p384_felem_mul(v, u1, i);
-
-  // x_out = r**2 - J - 2V
-  p384_felem_sqr(x_out, r);
-  p384_felem_sub(x_out, x_out, j);
-  p384_felem_sub(x_out, x_out, v);
-  p384_felem_sub(x_out, x_out, v);
-
-  // y_out = r(V-x_out) - 2 * s1 * J
-  p384_felem_sub(y_out, v, x_out);
-  p384_felem_mul(y_out, y_out, r);
-  p384_felem s1j;
-  p384_felem_mul(s1j, s1, j);
-  p384_felem_sub(y_out, y_out, s1j);
-  p384_felem_sub(y_out, y_out, s1j);
-
-  p384_felem_cmovznz(x_out, z1nz, x2, x_out);
-  p384_felem_cmovznz(x3, z2nz, x1, x_out);
-  p384_felem_cmovznz(y_out, z1nz, y2, y_out);
-  p384_felem_cmovznz(y3, z2nz, y1, y_out);
-  p384_felem_cmovznz(z_out, z1nz, z2, z_out);
-  p384_felem_cmovznz(z3, z2nz, z1, z_out);
+  ec_nistp_point_add(p384_methods(), x3, y3, z3, x1, y1, z1, mixed, x2, y2, z2);
 }
 
+#if defined(EC_NISTP_USE_S2N_BIGNUM)
+DEFINE_METHOD_FUNCTION(ec_nistp_meth, p384_methods) {
+    out->felem_num_limbs = P384_NLIMBS;
+    out->felem_add = bignum_add_p384;
+    out->felem_sub = bignum_sub_p384;
+    out->felem_mul = bignum_montmul_p384_selector;
+    out->felem_sqr = bignum_montsqr_p384_selector;
+    out->felem_nz  = p384_felem_nz;
+    out->point_dbl = p384_point_double;
+    out->point_add = p384_point_add;
+}
+#else
+DEFINE_METHOD_FUNCTION(ec_nistp_meth, p384_methods) {
+    out->felem_num_limbs = P384_NLIMBS;
+    out->felem_add = fiat_p384_add;
+    out->felem_sub = fiat_p384_sub;
+    out->felem_mul = fiat_p384_mul;
+    out->felem_sqr = fiat_p384_square;
+    out->felem_nz  = p384_felem_nz;
+    out->point_dbl = p384_point_double;
+    out->point_add = p384_point_add;
+}
+#endif
+
 // OPENSSL EC_METHOD FUNCTIONS
 
 // Takes the Jacobian coordinates (X, Y, Z) of a point and returns:
@@ -668,21 +470,6 @@ static int ec_GFp_nistp384_cmp_x_coordinate(const EC_GROUP *group,
 //
 // For detailed analysis of different window sizes see the bottom of this file.
 
-
-// p384_get_bit returns the |i|-th bit in |in|
-static crypto_word_t p384_get_bit(const EC_SCALAR *in, int i) {
-  if (i < 0 || i >= 384) {
-    return 0;
-  }
-#if defined(OPENSSL_64_BIT)
-  assert(sizeof(BN_ULONG) == 8);
-  return (in->words[i >> 6] >> (i & 63)) & 1;
-#else
-  assert(sizeof(BN_ULONG) == 4);
-  return (in->words[i >> 5] >> (i & 31)) & 1;
-#endif
-}
-
 // Constants for scalar encoding in the scalar multiplication functions.
 #define P384_MUL_WSIZE        (5) // window size w
 // Assert the window size is 5 because the pre-computed table in |p384_table.h|
@@ -691,7 +478,6 @@ OPENSSL_STATIC_ASSERT(P384_MUL_WSIZE == 5,
     p384_scalar_mul_window_size_is_not_equal_to_five)
 
 #define P384_MUL_TWO_TO_WSIZE (1 << P384_MUL_WSIZE)
-#define P384_MUL_WSIZE_MASK   ((P384_MUL_TWO_TO_WSIZE << 1) - 1)
 
 // Number of |P384_MUL_WSIZE|-bit windows in a 384-bit value
 #define P384_MUL_NWINDOWS     ((384 + P384_MUL_WSIZE - 1)/P384_MUL_WSIZE)
@@ -704,27 +490,6 @@ OPENSSL_STATIC_ASSERT(P384_MUL_WSIZE == 5,
 #define P384_MUL_TABLE_SIZE     (P384_MUL_TWO_TO_WSIZE >> 1)
 #define P384_MUL_PUB_TABLE_SIZE (1 << (P384_MUL_PUB_WSIZE - 1))
 
-// Compute "regular" wNAF representation of a scalar, see
-// Joye, Tunstall, "Exponent Recoding and Regular Exponentiation Algorithms",
-// AfricaCrypt 2009, Alg 6.
-// It forces an odd scalar and outputs digits in
-// {\pm 1, \pm 3, \pm 5, \pm 7, \pm 9, ...}
-// i.e. signed odd digits with _no zeroes_ -- that makes it "regular".
-static void p384_felem_mul_scalar_rwnaf(int16_t *out, const EC_SCALAR *in) {
-  int16_t window, d;
-
-  window = (in->words[0] & P384_MUL_WSIZE_MASK) | 1;
-  for (size_t i = 0; i < P384_MUL_NWINDOWS - 1; i++) {
-    d = (window & P384_MUL_WSIZE_MASK) - P384_MUL_TWO_TO_WSIZE;
-    out[i] = d;
-    window = (window - d) >> P384_MUL_WSIZE;
-    for (size_t j = 1; j <= P384_MUL_WSIZE; j++) {
-      window += p384_get_bit(in, (i + 1) * P384_MUL_WSIZE + j) << j;
-    }
-  }
-  out[P384_MUL_NWINDOWS - 1] = window;
-}
-
 // p384_select_point selects the |idx|-th projective point from the given
 // precomputed table and copies it to |out| in constant time.
 static void p384_select_point(p384_felem out[3],
@@ -812,7 +577,7 @@ static void ec_GFp_nistp384_point_mul(const EC_GROUP *group, EC_JACOBIAN *r,
 
   // Recode the scalar.
   int16_t rnaf[P384_MUL_NWINDOWS] = {0};
-  p384_felem_mul_scalar_rwnaf(rnaf, scalar);
+  scalar_rwnaf(rnaf, P384_MUL_WSIZE, scalar, 384);
 
   // Initialize the accumulator |res| with the table entry corresponding to
   // the most significant digit of the recoded scalar (note that this digit
@@ -936,7 +701,7 @@ static void ec_GFp_nistp384_point_mul_base(const EC_GROUP *group,
   int16_t rnaf[P384_MUL_NWINDOWS] = {0};
 
   // Recode the scalar.
-  p384_felem_mul_scalar_rwnaf(rnaf, scalar);
+  scalar_rwnaf(rnaf, P384_MUL_WSIZE, scalar, 384);
 
   // Process the 4 groups of digits starting from group (3) down to group (0).
   for (int i = 3; i >= 0; i--) {
