@@ -57,10 +57,15 @@
 
 #include <openssl/err.h>
 
-#include "internal.h"
 #include "../delocate.h"
 #include "../digest/internal.h"
+#include "internal.h"
 
+#if defined(NDEBUG)
+#define CHECK(x) (void) (x)
+#else
+#define CHECK(x) assert(x)
+#endif
 
 enum evp_sign_verify_t {
   evp_sign,
@@ -77,6 +82,28 @@ static int uses_prehash(EVP_MD_CTX *ctx, enum evp_sign_verify_t op) {
                           : (ctx->pctx->pmeth->verify != NULL);
 }
 
+static void hmac_update(EVP_MD_CTX *ctx, const void *data, size_t count) {
+  HMAC_PKEY_CTX *hctx = ctx->pctx->data;
+  CHECK(HMAC_Update(&hctx->ctx, data, count));
+}
+
+static int HMAC_DigestFinal_ex(EVP_MD_CTX *ctx, uint8_t *out_sig,
+                               size_t *out_sig_len) {
+  unsigned int mdlen;
+  if (*out_sig_len < EVP_MD_CTX_size(ctx)) {
+    OPENSSL_PUT_ERROR(EVP, EVP_R_BUFFER_TOO_SMALL);
+    return 0;
+  }
+
+  HMAC_PKEY_CTX *hctx = ctx->pctx->data;
+  if (!HMAC_Final(&hctx->ctx, out_sig, &mdlen)) {
+    return 0;
+  }
+
+  *out_sig_len = (size_t)mdlen;
+  return 1;
+}
+
 static int do_sigver_init(EVP_MD_CTX *ctx, EVP_PKEY_CTX **pctx,
                           const EVP_MD *type, ENGINE *e, EVP_PKEY *pkey,
                           enum evp_sign_verify_t op) {
@@ -93,17 +120,26 @@ static int do_sigver_init(EVP_MD_CTX *ctx, EVP_PKEY_CTX **pctx,
       return 0;
     }
   } else {
-    if (!EVP_PKEY_sign_init(ctx->pctx)) {
-      return 0;
+    if (pkey->type == EVP_PKEY_HMAC) {
+      // |ctx->update| gets repurposed as a hook to call |HMAC_Update|.
+      // |ctx->update| is normally copied from |ctx->digest->update|, but
+      // |EVP_PKEY_HMAC| has its own definition. We suppress the automatic
+      // setting of |mctx->update| and the rest of its initialization here.
+      ctx->pctx->operation = EVP_PKEY_OP_SIGN;
+      ctx->flags |= EVP_MD_CTX_HMAC;
+      ctx->update = hmac_update;
+    } else {
+      if (!EVP_PKEY_sign_init(ctx->pctx)) {
+        return 0;
+      }
     }
   }
 
-  if (type != NULL &&
-      !EVP_PKEY_CTX_set_signature_md(ctx->pctx, type)) {
+  if (type != NULL && !EVP_PKEY_CTX_set_signature_md(ctx->pctx, type)) {
     return 0;
   }
 
-  if (uses_prehash(ctx, op)) {
+  if (uses_prehash(ctx, op) || used_for_hmac(ctx)) {
     if (type == NULL) {
       OPENSSL_PUT_ERROR(EVP, EVP_R_NO_DEFAULT_DIGEST);
       return 0;
@@ -130,7 +166,7 @@ int EVP_DigestVerifyInit(EVP_MD_CTX *ctx, EVP_PKEY_CTX **pctx,
 }
 
 int EVP_DigestSignUpdate(EVP_MD_CTX *ctx, const void *data, size_t len) {
-  if (!uses_prehash(ctx, evp_sign)) {
+  if (!uses_prehash(ctx, evp_sign) && !used_for_hmac(ctx)) {
     OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
     return 0;
   }
@@ -139,7 +175,7 @@ int EVP_DigestSignUpdate(EVP_MD_CTX *ctx, const void *data, size_t len) {
 }
 
 int EVP_DigestVerifyUpdate(EVP_MD_CTX *ctx, const void *data, size_t len) {
-  if (!uses_prehash(ctx, evp_verify)) {
+  if (!uses_prehash(ctx, evp_verify) || used_for_hmac(ctx)) {
     OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
     return 0;
   }
@@ -149,28 +185,32 @@ int EVP_DigestVerifyUpdate(EVP_MD_CTX *ctx, const void *data, size_t len) {
 
 int EVP_DigestSignFinal(EVP_MD_CTX *ctx, uint8_t *out_sig,
                         size_t *out_sig_len) {
-  if (!uses_prehash(ctx, evp_sign)) {
+  if (!uses_prehash(ctx, evp_sign) && !used_for_hmac(ctx)) {
     OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
     return 0;
   }
 
   if (out_sig) {
     EVP_MD_CTX tmp_ctx;
-    int ret;
+    int ret = 0;
     uint8_t md[EVP_MAX_MD_SIZE];
     unsigned int mdlen;
     // We have to avoid the underlying SHA services updating the indicator
     // state, so we lock the state here.
     FIPS_service_indicator_lock_state();
-
     EVP_MD_CTX_init(&tmp_ctx);
-    ret = EVP_MD_CTX_copy_ex(&tmp_ctx, ctx) &&
-          EVP_DigestFinal_ex(&tmp_ctx, md, &mdlen) &&
-          EVP_PKEY_sign(ctx->pctx, out_sig, out_sig_len, md, mdlen);
-    EVP_MD_CTX_cleanup(&tmp_ctx);
+    if (EVP_MD_CTX_copy_ex(&tmp_ctx, ctx)) {
+      if (used_for_hmac(ctx)) {
+        ret = HMAC_DigestFinal_ex(&tmp_ctx, out_sig, out_sig_len);
+      } else {
+        ret = EVP_DigestFinal_ex(&tmp_ctx, md, &mdlen) &&
+              EVP_PKEY_sign(ctx->pctx, out_sig, out_sig_len, md, mdlen);
+      }
+    }
 
+    EVP_MD_CTX_cleanup(&tmp_ctx);
     FIPS_service_indicator_unlock_state();
-    if(ret > 0) {
+    if (ret > 0) {
       EVP_DigestSign_verify_service_indicator(ctx);
     }
     return ret;
@@ -178,14 +218,19 @@ int EVP_DigestSignFinal(EVP_MD_CTX *ctx, uint8_t *out_sig,
     // This only determines the size of the signature. This case of
     // |EVP_DigestSignFinal| should not return an approval check because no
     // crypto is being done.
-    size_t s = EVP_MD_size(ctx->digest);
-    return EVP_PKEY_sign(ctx->pctx, out_sig, out_sig_len, NULL, s);
+    if (used_for_hmac(ctx)) {
+      // This is only defined in |EVP_PKEY_HMAC|.
+      *out_sig_len = EVP_MD_CTX_size(ctx);
+      return 1;
+    } else {
+      size_t s = EVP_MD_size(ctx->digest);
+      return EVP_PKEY_sign(ctx->pctx, out_sig, out_sig_len, NULL, s);
+    }
   }
 }
 
-int EVP_DigestVerifyFinal(EVP_MD_CTX *ctx, const uint8_t *sig,
-                          size_t sig_len) {
-  if (!uses_prehash(ctx, evp_verify)) {
+int EVP_DigestVerifyFinal(EVP_MD_CTX *ctx, const uint8_t *sig, size_t sig_len) {
+  if (!uses_prehash(ctx, evp_verify) || used_for_hmac(ctx)) {
     OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
     return 0;
   }
@@ -205,7 +250,7 @@ int EVP_DigestVerifyFinal(EVP_MD_CTX *ctx, const uint8_t *sig,
   EVP_MD_CTX_cleanup(&tmp_ctx);
 
   FIPS_service_indicator_unlock_state();
-  if(ret > 0) {
+  if (ret > 0) {
     EVP_DigestVerify_verify_service_indicator(ctx);
   }
   return ret;
@@ -218,11 +263,10 @@ int EVP_DigestSign(EVP_MD_CTX *ctx, uint8_t *out_sig, size_t *out_sig_len,
   FIPS_service_indicator_lock_state();
   int ret = 0;
 
-  if (uses_prehash(ctx, evp_sign)) {
+  if (uses_prehash(ctx, evp_sign) || used_for_hmac(ctx)) {
     // If |out_sig| is NULL, the caller is only querying the maximum output
     // length. |data| should only be incorporated in the final call.
-    if (out_sig != NULL &&
-        !EVP_DigestSignUpdate(ctx, data, data_len)) {
+    if (out_sig != NULL && !EVP_DigestSignUpdate(ctx, data, data_len)) {
       goto end;
     }
 
@@ -238,10 +282,10 @@ int EVP_DigestSign(EVP_MD_CTX *ctx, uint8_t *out_sig, size_t *out_sig_len,
   // This is executed when |uses_prehash| is not true, which is the case for
   // Ed25519 and Dilithium.
   ret = ctx->pctx->pmeth->sign_message(ctx->pctx, out_sig, out_sig_len, data,
-                                        data_len);
+                                       data_len);
 end:
   FIPS_service_indicator_unlock_state();
-  if(ret > 0) {
+  if (ret > 0) {
     EVP_DigestSign_verify_service_indicator(ctx);
   }
   return ret;
@@ -254,7 +298,7 @@ int EVP_DigestVerify(EVP_MD_CTX *ctx, const uint8_t *sig, size_t sig_len,
   FIPS_service_indicator_lock_state();
   int ret = 0;
 
-  if (uses_prehash(ctx, evp_verify)) {
+  if (uses_prehash(ctx, evp_verify) && !used_for_hmac(ctx)) {
     ret = EVP_DigestVerifyUpdate(ctx, data, len) &&
           EVP_DigestVerifyFinal(ctx, sig, sig_len);
     goto end;
@@ -271,7 +315,7 @@ int EVP_DigestVerify(EVP_MD_CTX *ctx, const uint8_t *sig, size_t sig_len,
 
 end:
   FIPS_service_indicator_unlock_state();
-  if(ret > 0) {
+  if (ret > 0) {
     EVP_DigestVerify_verify_service_indicator(ctx);
   }
   return ret;
