@@ -75,17 +75,16 @@
 typedef struct {
   // message digest
   const EVP_MD *md;
-  EC_GROUP *gen_group;
+  const EC_GROUP *gen_group;
 } EC_PKEY_CTX;
 
 
 static int pkey_ec_init(EVP_PKEY_CTX *ctx) {
   EC_PKEY_CTX *dctx;
-  dctx = OPENSSL_malloc(sizeof(EC_PKEY_CTX));
+  dctx = OPENSSL_zalloc(sizeof(EC_PKEY_CTX));
   if (!dctx) {
     return 0;
   }
-  OPENSSL_memset(dctx, 0, sizeof(EC_PKEY_CTX));
 
   ctx->data = dctx;
 
@@ -111,7 +110,6 @@ static void pkey_ec_cleanup(EVP_PKEY_CTX *ctx) {
     return;
   }
 
-  EC_GROUP_free(dctx->gen_group);
   OPENSSL_free(dctx);
 }
 
@@ -214,11 +212,10 @@ static int pkey_ec_ctrl(EVP_PKEY_CTX *ctx, int type, int p1, void *p2) {
       return 1;
 
     case EVP_PKEY_CTRL_EC_PARAMGEN_CURVE_NID: {
-      EC_GROUP *group = EC_GROUP_new_by_curve_name(p1);
+      const EC_GROUP *group = EC_GROUP_new_by_curve_name(p1);
       if (group == NULL) {
         return 0;
       }
-      EC_GROUP_free(dctx->gen_group);
       dctx->gen_group = group;
       return 1;
     }
