@@ -126,3 +126,9 @@ FILE* createRawTempFILE() {
 TempFILE createTempFILE() {
   return TempFILE(createRawTempFILE());
 }
+
+void CustomDataFree(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
+                           int index, long argl, void *argp) {
+  free(ptr);
+}
+
