@@ -233,10 +233,10 @@ impl<'a, T: 'a + Context> Callback<'a, T> {
 #[cfg(test)]
 mod tests {
     use crate::{
-        callbacks::{ClientHelloCallback, ConnectionFuture},
+        callbacks::{ClientHelloCallback, ConnectionFuture, ConnectionFutureResult},
         enums::ClientAuthType,
         error::ErrorType,
-        testing::{client_hello::*, s2n_tls::*, *},
+        testing::{client_hello::*, *},
     };
     use alloc::sync::Arc;
     use core::sync::atomic::Ordering;
@@ -246,13 +246,13 @@ mod tests {
     #[test]
     fn handshake_default() {
         let config = build_config(&security::DEFAULT).unwrap();
-        establish_connection(config);
+        assert!(TestPair::handshake_with_config(&config).is_ok());
     }
 
     #[test]
     fn handshake_default_tls13() {
         let config = build_config(&security::DEFAULT_TLS13).unwrap();
-        establish_connection(config)
+        assert!(TestPair::handshake_with_config(&config).is_ok());
     }
 
     #[test]
@@ -343,43 +343,19 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            // create and configure a server connection
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            server.set_waker(Some(&waker))?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.server.set_waker(Some(&waker))?;
+        let s2n_err = pair.handshake().unwrap_err();
+        // the underlying error should be the custom error the application provided
+        let app_err = s2n_err.application_error().unwrap();
+        let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
+        let _custom_err = io_err
+            .get_ref()
+            .unwrap()
+            .downcast_ref::<CustomError>()
+            .unwrap();
 
-        let client = {
-            // create a client connection
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let mut pair = Pair::new(server, client);
-        loop {
-            match pair.poll() {
-                Poll::Ready(result) => {
-                    let err = result.expect_err("handshake should fail");
-
-                    // the underlying error should be the custom error the application provided
-                    let s2n_err = err.downcast_ref::<crate::error::Error>().unwrap();
-                    let app_err = s2n_err.application_error().unwrap();
-                    let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
-                    let _custom_err = io_err
-                        .get_ref()
-                        .unwrap()
-                        .downcast_ref::<CustomError>()
-                        .unwrap();
-                    break;
-                }
-                Poll::Pending => continue,
-            }
-        }
         assert_eq!(wake_count, 0);
-
         Ok(())
     }
 
@@ -392,44 +368,20 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            // create and configure a server connection
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            server.set_waker(Some(&waker))?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.server.set_waker(Some(&waker))?;
+        let s2n_err = pair.handshake().unwrap_err();
+        // the underlying error should be the custom error the application provided
+        let app_err = s2n_err.application_error().unwrap();
+        let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
+        let _custom_err = io_err
+            .get_ref()
+            .unwrap()
+            .downcast_ref::<CustomError>()
+            .unwrap();
 
-        let client = {
-            // create a client connection
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let mut pair = Pair::new(server, client);
-        loop {
-            match pair.poll() {
-                Poll::Ready(result) => {
-                    let err = result.expect_err("handshake should fail");
-
-                    // the underlying error should be the custom error the application provided
-                    let s2n_err = err.downcast_ref::<crate::error::Error>().unwrap();
-                    let app_err = s2n_err.application_error().unwrap();
-                    let io_err = app_err.downcast_ref::<std::io::Error>().unwrap();
-                    let _custom_err = io_err
-                        .get_ref()
-                        .unwrap()
-                        .downcast_ref::<CustomError>()
-                        .unwrap();
-                    break;
-                }
-                Poll::Pending => continue,
-            }
-        }
         // assert that the future is async returned Poll::Pending once
         assert_eq!(wake_count, 1);
-
         Ok(())
     }
 
@@ -446,24 +398,10 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            // create and configure a server connection
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            server.set_waker(Some(&waker))?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.server.set_waker(Some(&waker))?;
+        pair.handshake()?;
 
-        let client = {
-            // create a client connection
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let pair = Pair::new(server, client);
-
-        poll_tls_pair(pair);
         // confirm that the callback returned Pending `require_pending_count` times
         assert_eq!(wake_count, require_pending_count);
         // confirm that the final invoked count is +1 more than `require_pending_count`
@@ -474,6 +412,7 @@ mod tests {
 
         Ok(())
     }
+
     #[test]
     fn client_hello_callback_sync() -> Result<(), Error> {
         let (waker, wake_count) = new_count_waker();
@@ -515,25 +454,12 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            // create and configure a server connection
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            server.set_waker(Some(&waker))?;
-            Harness::new(server)
-        };
-
-        let client = {
-            // create a client connection
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let pair = Pair::new(server, client);
+        let mut pair = TestPair::from_config(&config);
+        pair.server.set_waker(Some(&waker))?;
 
         assert_eq!(callback.count(), 0);
-        poll_tls_pair(pair);
+
+        pair.handshake()?;
         assert_eq!(callback.count(), 1);
         assert_eq!(wake_count, 0);
         Ok(())
@@ -562,7 +488,7 @@ mod tests {
         builder.load_pem(&fs::read(&cert)?, &fs::read(&key)?)?;
         builder.trust_location(Some(&cert), None)?;
 
-        establish_connection(builder.build()?);
+        TestPair::handshake_with_config(&builder.build()?)?;
         Ok(())
     }
 
@@ -600,13 +526,9 @@ mod tests {
                 config.build()?
             };
 
-            let mut pair = tls_pair(config);
-            pair.server
-                .0
-                .connection_mut()
-                .set_waker(Some(&noop_waker()))?;
-
-            poll_tls_pair(pair);
+            let mut pair = TestPair::from_config(&config);
+            pair.server.set_waker(Some(&noop_waker()))?;
+            pair.handshake()?;
         }
         Ok(())
     }
@@ -626,23 +548,17 @@ mod tests {
         };
 
         // confirm that default connection establishment fails
-        let mut pair = tls_pair(reject_config.clone());
-        assert!(poll_tls_pair_result(&mut pair).is_err());
+        let mut pair = TestPair::from_config(&reject_config);
+        assert!(pair.handshake().is_err());
 
         // confirm that overriding the verify_host_callback on connection causes
         // the handshake to succeed
-        pair = tls_pair(reject_config);
-        pair.server
-            .0
-            .connection
-            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
-            .unwrap();
+        let mut pair = TestPair::from_config(&reject_config);
         pair.client
-            .0
-            .connection
-            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
-            .unwrap();
-        assert!(poll_tls_pair_result(&mut pair).is_ok());
+            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})?;
+        pair.server
+            .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})?;
+        pair.handshake()?;
 
         Ok(())
     }
@@ -657,24 +573,10 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.handshake()?;
 
-        let client = {
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let pair = Pair::new(server, client);
-        let pair = poll_tls_pair(pair);
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
-
-        for conn in [server, client] {
+        for conn in [pair.server, pair.client] {
             assert!(!conn.client_cert_used());
             let cert = conn.client_cert_chain_bytes()?;
             assert!(cert.is_none());
@@ -697,28 +599,14 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.handshake()?;
 
-        let client = {
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let pair = Pair::new(server, client);
-        let pair = poll_tls_pair(pair);
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
-
-        let cert = server.client_cert_chain_bytes()?;
+        let cert = pair.server.client_cert_chain_bytes()?;
         assert!(cert.is_some());
         assert!(!cert.unwrap().is_empty());
 
-        for conn in [server, client] {
+        for conn in [pair.server, pair.client] {
             assert!(conn.client_cert_used());
             let sig_alg = conn.selected_client_signature_algorithm()?;
             assert!(sig_alg.is_some());
@@ -730,7 +618,7 @@ mod tests {
     }
 
     #[test]
-    fn system_certs_loaded_by_default() {
+    fn system_certs_loaded_by_default() -> Result<(), Error> {
         let keypair = CertKeyPair::default();
 
         // Load the server certificate into the trust store by overriding the OpenSSL default
@@ -738,20 +626,18 @@ mod tests {
         temp_env::with_var("SSL_CERT_FILE", Some(keypair.cert_path()), || {
             let mut builder = Builder::new();
             builder
-                .load_pem(keypair.cert(), keypair.key())
-                .unwrap()
-                .set_security_policy(&security::DEFAULT_TLS13)
-                .unwrap()
-                .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
-                .unwrap();
+                .load_pem(keypair.cert(), keypair.key())?
+                .set_security_policy(&security::DEFAULT_TLS13)?
+                .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})?;
 
             let config = builder.build().unwrap();
-            establish_connection(config);
-        });
+            TestPair::handshake_with_config(&config)?;
+            Ok(())
+        })
     }
 
     #[test]
-    fn disable_loading_system_certs() {
+    fn disable_loading_system_certs() -> Result<(), Error> {
         let keypair = CertKeyPair::default();
 
         // Load the server certificate into the trust store by overriding the OpenSSL default
@@ -760,24 +646,19 @@ mod tests {
             // Test the Builder itself, and also the Builder produced by the Config builder() API.
             for mut builder in [Builder::new(), Config::builder()] {
                 builder
-                    .load_pem(keypair.cert(), keypair.key())
-                    .unwrap()
-                    .set_security_policy(&security::DEFAULT_TLS13)
-                    .unwrap()
-                    .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
-                    .unwrap();
+                    .load_pem(keypair.cert(), keypair.key())?
+                    .set_security_policy(&security::DEFAULT_TLS13)?
+                    .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})?;
 
                 // Disable loading system certificates
-                builder.with_system_certs(false).unwrap();
+                builder.with_system_certs(false)?;
 
-                let config = builder.build().unwrap();
+                let config = builder.build()?;
                 let mut config_with_system_certs = config.clone();
 
-                let mut pair = tls_pair(config);
-
                 // System certificates should not be loaded into the trust store. The handshake
                 // should fail since the certificate should not be trusted.
-                assert!(poll_tls_pair_result(&mut pair).is_err());
+                assert!(TestPair::handshake_with_config(&config).is_err());
 
                 // The handshake should succeed after trusting the certificate.
                 unsafe {
@@ -785,9 +666,10 @@ mod tests {
                         config_with_system_certs.as_mut_ptr(),
                     );
                 }
-                establish_connection(config_with_system_certs);
+                TestPair::handshake_with_config(&config_with_system_certs)?;
             }
-        });
+            Ok(())
+        })
     }
 
     #[test]
@@ -800,24 +682,10 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            Harness::new(server)
-        };
+        let mut pair = TestPair::from_config(&config);
+        pair.handshake()?;
 
-        let client = {
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
-
-        let pair = Pair::new(server, client);
-        let pair = poll_tls_pair(pair);
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
-
-        for conn in [server, client] {
+        for conn in [pair.server, pair.client] {
             let chain = conn.peer_cert_chain()?;
             assert_eq!(chain.len(), 1);
             for cert in chain.iter() {
@@ -840,29 +708,15 @@ mod tests {
             config.build()?
         };
 
-        let server = {
-            let mut server = crate::connection::Connection::new_server();
-            server.set_config(config.clone())?;
-            Harness::new(server)
-        };
-
-        let client = {
-            let mut client = crate::connection::Connection::new_client();
-            client.set_config(config)?;
-            Harness::new(client)
-        };
+        let mut pair = TestPair::from_config(&config);
 
         // None before handshake...
-        assert!(server.connection.selected_cert().is_none());
-        assert!(client.connection.selected_cert().is_none());
+        assert!(pair.server.selected_cert().is_none());
+        assert!(pair.client.selected_cert().is_none());
 
-        let pair = Pair::new(server, client);
+        pair.handshake()?;
 
-        let pair = poll_tls_pair(pair);
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
-
-        for conn in [&server, &client] {
+        for conn in [&pair.server, &pair.client] {
             let chain = conn.selected_cert().unwrap();
             assert_eq!(chain.len(), 1);
             for cert in chain.iter() {
@@ -875,14 +729,14 @@ mod tests {
         // Same config is used for both and we are doing mTLS, so both should select the same
         // certificate.
         assert_eq!(
-            server
+            pair.server
                 .selected_cert()
                 .unwrap()
                 .iter()
                 .next()
                 .unwrap()?
                 .der()?,
-            client
+            pair.client
                 .selected_cert()
                 .unwrap()
                 .iter()
@@ -898,12 +752,11 @@ mod tests {
     fn master_secret_success() -> Result<(), Error> {
         let policy = security::Policy::from_version("test_all_tls12")?;
         let config = config_builder(&policy)?.build()?;
-        let pair = poll_tls_pair(tls_pair(config));
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
+        let mut pair = TestPair::from_config(&config);
+        pair.handshake()?;
 
-        let server_secret = server.master_secret()?;
-        let client_secret = client.master_secret()?;
+        let server_secret = pair.server.master_secret()?;
+        let client_secret = pair.client.master_secret()?;
         assert_eq!(server_secret, client_secret);
 
         Ok(())
@@ -912,16 +765,13 @@ mod tests {
     #[test]
     fn master_secret_failure() -> Result<(), Error> {
         // TLS1.3 does not support getting the master secret
-        let config = config_builder(&security::DEFAULT_TLS13)?.build()?;
-        let pair = poll_tls_pair(tls_pair(config));
-        let server = pair.server.0.connection;
-        let client = pair.client.0.connection;
+        let mut pair = TestPair::from_config(&build_config(&security::DEFAULT_TLS13)?);
+        pair.handshake()?;
 
-        let server_error = server.master_secret().unwrap_err();
-        assert_eq!(server_error.kind(), ErrorType::UsageError);
-
-        let client_error = client.master_secret().unwrap_err();
-        assert_eq!(client_error.kind(), ErrorType::UsageError);
+        for conn in [pair.client, pair.server] {
+            let err = conn.master_secret().unwrap_err();
+            assert_eq!(err.kind(), ErrorType::UsageError);
+        }
 
         Ok(())
     }
@@ -936,26 +786,21 @@ mod tests {
             send_key_updates: 0,
         };
 
-        let pair = tls_pair(build_config(&security::DEFAULT_TLS13)?);
-        let mut pair = poll_tls_pair(pair);
+        let mut pair = TestPair::from_config(&build_config(&security::DEFAULT_TLS13)?);
+        pair.handshake()?;
 
         // there haven't been any key updates at the start of the connection
-        let client_updates = pair.client.0.connection.as_ref().key_update_counts()?;
-        assert_eq!(client_updates, empty_key_updates);
-        let server_updates = pair.server.0.connection.as_ref().key_update_counts()?;
-        assert_eq!(server_updates, empty_key_updates);
+        assert_eq!(pair.client.key_update_counts()?, empty_key_updates);
+        assert_eq!(pair.server.key_update_counts()?, empty_key_updates);
 
         pair.server
-            .0
-            .connection
-            .as_mut()
             .request_key_update(PeerKeyUpdate::KeyUpdateNotRequested)?;
-        assert!(pair.poll_send(Mode::Server, &[0]).is_ready());
+        assert!(pair.server.poll_send(&[0]).is_ready());
 
         // the server send key has been updated
-        let client_updates = pair.client.0.connection.as_ref().key_update_counts()?;
+        let client_updates = pair.client.key_update_counts()?;
         assert_eq!(client_updates, empty_key_updates);
-        let server_updates = pair.server.0.connection.as_ref().key_update_counts()?;
+        let server_updates = pair.server.key_update_counts()?;
         assert_eq!(server_updates.recv_key_updates, 0);
         assert_eq!(server_updates.send_key_updates, 1);
 
@@ -970,4 +815,83 @@ mod tests {
         init::init();
         assert!(init::fips_mode().unwrap().is_enabled());
     }
+
+    /// Test that a context can be used from within a callback.
+    #[test]
+    fn test_app_context_callback() -> Result<(), crate::error::Error> {
+        struct TestApplicationContext {
+            invoked_count: u32,
+        }
+
+        struct TestClientHelloHandler {}
+
+        impl ClientHelloCallback for TestClientHelloHandler {
+            fn on_client_hello(
+                &self,
+                connection: &mut connection::Connection,
+            ) -> ConnectionFutureResult {
+                let app_context = connection
+                    .application_context_mut::<TestApplicationContext>()
+                    .unwrap();
+                app_context.invoked_count += 1;
+                Ok(None)
+            }
+        }
+
+        let config = {
+            let keypair = CertKeyPair::default();
+            let mut builder = Builder::new();
+            builder
+                .set_verify_host_callback(InsecureAcceptAllCertificatesHandler {})
+                .unwrap();
+            builder
+                .set_client_hello_callback(TestClientHelloHandler {})
+                .unwrap();
+            builder.load_pem(keypair.cert, keypair.key).unwrap();
+            builder.trust_pem(keypair.cert).unwrap();
+            builder.build().unwrap()
+        };
+        let mut pair = TestPair::from_config(&config);
+        pair.server.set_waker(Some(&noop_waker()))?;
+
+        let context = TestApplicationContext { invoked_count: 0 };
+        pair.server.set_application_context(context);
+
+        pair.handshake()?;
+
+        let context = pair
+            .server
+            .application_context::<TestApplicationContext>()
+            .unwrap();
+        assert_eq!(context.invoked_count, 1);
+
+        Ok(())
+    }
+
+    #[test]
+    fn no_application_protocol() -> Result<(), Error> {
+        let config = config_builder(&security::DEFAULT)?.build()?;
+        let mut pair = tls_pair(config);
+        assert!(poll_tls_pair_result(&mut pair).is_ok());
+        assert!(pair.server.0.connection.application_protocol().is_none());
+        Ok(())
+    }
+
+    #[test]
+    fn application_protocol() -> Result<(), Error> {
+        let config = config_builder(&security::DEFAULT)?.build()?;
+        let mut pair = tls_pair(config);
+        pair.server
+            .0
+            .connection
+            .set_application_protocol_preference(["http/1.1", "h2"])?;
+        pair.client
+            .0
+            .connection
+            .append_application_protocol_preference(b"h2")?;
+        assert!(poll_tls_pair_result(&mut pair).is_ok());
+        let protocol = pair.server.0.connection.application_protocol().unwrap();
+        assert_eq!(protocol, b"h2");
+        Ok(())
+    }
 }
