@@ -364,7 +364,8 @@ PyObject *PyErr_AwsLastError(void) {
 }
 
 #define AWS_DEFINE_ERROR_INFO_CRT(CODE, STR)                                                                           \
-    [(CODE)-AWS_ERROR_ENUM_BEGIN_RANGE(AWS_CRT_PYTHON_PACKAGE_ID)] = AWS_DEFINE_ERROR_INFO(CODE, STR, "aws-crt-python")
+    [(CODE) - AWS_ERROR_ENUM_BEGIN_RANGE(AWS_CRT_PYTHON_PACKAGE_ID)] =                                                 \
+        AWS_DEFINE_ERROR_INFO(CODE, STR, "aws-crt-python")
 
 /* clang-format off */
 static struct aws_error_info s_errors[] = {
@@ -655,8 +656,7 @@ static void s_install_crash_handler(void) {
  * Definitions
  ******************************************************************************/
 
-#define AWS_PY_METHOD_DEF(NAME, FLAGS)                                                                                 \
-    { #NAME, aws_py_##NAME, (FLAGS), NULL }
+#define AWS_PY_METHOD_DEF(NAME, FLAGS) {#NAME, aws_py_##NAME, (FLAGS), NULL}
 
 static PyMethodDef s_module_methods[] = {
     /* Common */
@@ -802,6 +802,7 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(s3_get_ec2_instance_type, METH_NOARGS),
     AWS_PY_METHOD_DEF(s3_is_crt_s3_optimized_for_system, METH_NOARGS),
     AWS_PY_METHOD_DEF(s3_get_recommended_throughput_target_gbps, METH_NOARGS),
+    AWS_PY_METHOD_DEF(s3_get_optimized_platforms, METH_NOARGS),
     AWS_PY_METHOD_DEF(s3_cross_process_lock_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(s3_cross_process_lock_acquire, METH_VARARGS),
     AWS_PY_METHOD_DEF(s3_cross_process_lock_release, METH_VARARGS),
