@@ -14,9 +14,7 @@ from concurrent.futures import Future
 from multiprocessing import Process
 
 from awscrt.http import HttpHeaders, HttpRequest
-from awscrt.s3 import S3Client, S3RequestType, create_default_s3_signing_config
-from awscrt.io import ClientBootstrap, ClientTlsContext, DefaultHostResolver, EventLoopGroup, TlsConnectionOptions, TlsContextOptions
-from awscrt.auth import AwsCredentials, AwsCredentialsProvider, AwsSignatureType, AwsSignedBodyHeaderType, AwsSignedBodyValue, AwsSigningAlgorithm, AwsSigningConfig
+from awscrt.auth import AwsCredentials
 from awscrt.s3 import (
     S3ChecksumAlgorithm,
     S3ChecksumConfig,
@@ -90,7 +88,7 @@ class CrossProcessLockTest(NativeResourceTest):
             process = Process(target=cross_proc_task)
             process.start()
             process.join()
-            # aquiring this lock in a sub-process should fail since we
+            # acquiring this lock in a sub-process should fail since we
             # already hold the lock in this process.
             self.assertNotEqual(0, process.exitcode)
 
@@ -158,7 +156,8 @@ def s3_client_new(
         part_size=0,
         is_cancel_test=False,
         enable_s3express=False,
-        mem_limit=None):
+        mem_limit=None,
+        network_interface_names=None):
 
     if is_cancel_test:
         # for cancellation tests, make things slow, so it's less likely that
@@ -189,7 +188,8 @@ def s3_client_new(
         part_size=part_size,
         throughput_target_gbps=throughput_target_gbps,
         enable_s3express=enable_s3express,
-        memory_limit=mem_limit)
+        memory_limit=mem_limit,
+        network_interface_names=network_interface_names)
 
     return s3_client
 
@@ -221,6 +221,10 @@ class S3ClientTest(NativeResourceTest):
         s3_client = s3_client_new(True, self.region)
         self.assertIsNotNone(s3_client)
 
+    def test_sanity_network_interface_names(self):
+        s3_client = s3_client_new(True, self.region, network_interface_names=("eth0", "eth1"))
+        self.assertIsNotNone(s3_client)
+
     def test_wait_shutdown(self):
         s3_client = s3_client_new(False, self.region)
         self.assertIsNotNone(s3_client)
@@ -260,6 +264,8 @@ class S3RequestTest(NativeResourceTest):
         self.done_error_headers = None
         self.done_error_body = None
         self.done_error_operation_name = None
+        self.done_did_validate_checksum = None
+        self.done_checksum_validation_algorithm = None
 
         self.files = FileCreator()
         self.temp_put_obj_file_path = self.files.create_file_with_size("temp_put_obj_10mb", 10 * MB)
@@ -305,12 +311,23 @@ class S3RequestTest(NativeResourceTest):
     def _on_request_body(self, chunk, offset, **kargs):
         self.received_body_len = self.received_body_len + len(chunk)
 
-    def _on_request_done(self, error, error_headers, error_body, error_operation_name, status_code, **kwargs):
+    def _on_request_done(
+            self,
+            error,
+            error_headers,
+            error_body,
+            error_operation_name,
+            status_code,
+            did_validate_checksum,
+            checksum_validation_algorithm,
+            **kwargs):
         self.done_error = error
         self.done_error_headers = error_headers
         self.done_error_body = error_body
         self.done_error_operation_name = error_operation_name
         self.done_status_code = status_code
+        self.done_did_validate_checksum = did_validate_checksum
+        self.done_checksum_validation_algorithm = checksum_validation_algorithm
 
     def _on_progress(self, progress):
         self.transferred_len += progress
@@ -323,6 +340,11 @@ class S3RequestTest(NativeResourceTest):
         self.assertIsNone(self.done_error_headers)
         self.assertIsNone(self.done_error_body)
         self.assertIsNone(self.done_error_operation_name)
+        self.assertIsInstance(self.done_did_validate_checksum, bool)
+        if self.done_did_validate_checksum:
+            self.assertIsInstance(self.done_checksum_validation_algorithm, S3ChecksumAlgorithm)
+        else:
+            self.assertIsNone(self.done_checksum_validation_algorithm)
         headers = HttpHeaders(self.response_headers)
         self.assertIsNone(headers.get("Content-Range"))
         body_length = headers.get("Content-Length")
@@ -452,6 +474,38 @@ class S3RequestTest(NativeResourceTest):
         del s3_client
         self.assertTrue(client_shutdown_event.wait(self.timeout))
 
+    def test_put_object_request_override_part_size(self):
+        s3_client = s3_client_new(False, self.region, 5 * MB)
+
+        tempfile = self.files.create_file_with_size("temp_file_override", 10 * MB)
+        path = "/put_object_test_py_10MB_override.txt"
+        content_length = os.stat(tempfile).st_size
+        request = self._put_object_request(None, content_length, path=path)
+        # Override the threshold to 10 MB, which will result in a single part upload
+        s3_request = s3_client.make_request(
+            request=request,
+            type=S3RequestType.PUT_OBJECT,
+            send_filepath=tempfile,
+            on_headers=self._on_request_headers,
+            on_body=self._on_request_body,
+            on_done=self._on_request_done,
+            multipart_upload_threshold=10 * MB)
+        try:
+            s3_request.finished_future.result(self.timeout)
+        except Exception as e:
+            # failed
+            self.assertTrue(False)
+
+        # Etag headers for a MPU will be formatted with `-[part number]`
+        etag = HttpHeaders(self.response_headers).get("Etag")
+        # make sure we uploaded as single part as we override the threshold
+        self.assertFalse("-" in etag)
+
+        del s3_request
+        client_shutdown_event = s3_client.shutdown_event
+        del s3_client
+        self.assertTrue(client_shutdown_event.wait(self.timeout))
+
     def test_get_object_filepath(self):
         request = self._get_object_request(self.get_test_object_path)
         request_type = S3RequestType.GET_OBJECT
@@ -559,6 +613,8 @@ class S3RequestTest(NativeResourceTest):
         download_checksum_config = S3ChecksumConfig(validate_response=True)
         self._test_s3_put_get_object(download_request, S3RequestType.GET_OBJECT,
                                      checksum_config=download_checksum_config)
+        self.assertTrue(self.done_did_validate_checksum)
+        self.assertEqual(self.done_checksum_validation_algorithm, S3ChecksumAlgorithm.CRC32)
         self.assertEqual(HttpHeaders(self.response_headers).get('x-amz-checksum-crc32'),
                          crc32_base64_str)
 
